from _typeshed import Incomplete

from antlr4.Recognizer import Recognizer as Recognizer
from antlr4.RuleContext import RuleContext as RuleContext

class SemanticContext:
    NONE: Incomplete
    def eval(self, parser: Recognizer, outerContext: RuleContext): ...
    def evalPrecedence(self, parser: Recognizer, outerContext: RuleContext): ...

def andContext(a: SemanticContext, b: SemanticContext): ...
def orContext(a: SemanticContext, b: SemanticContext): ...
def filterPrecedencePredicates(collection: set[SemanticContext]): ...

class EmptySemanticContext(SemanticContext): ...

class Predicate(SemanticContext):
    ruleIndex: Incomplete
    predIndex: Incomplete
    isCtxDependent: Incomplete
    def __init__(self, ruleIndex: int = -1, predIndex: int = -1, isCtxDependent: bool = False) -> None: ...
    def eval(self, parser: Recognizer, outerContext: RuleContext): ...
    def __hash__(self): ...
    def __eq__(self, other): ...

class PrecedencePredicate(SemanticContext):
    precedence: Incomplete
    def __init__(self, precedence: int = 0) -> None: ...
    def eval(self, parser: Recognizer, outerContext: RuleContext): ...
    def evalPrecedence(self, parser: Recognizer, outerContext: RuleContext): ...
    def __lt__(self, other): ...
    def __hash__(self): ...
    def __eq__(self, other): ...

class AND(SemanticContext):
    opnds: Incomplete
    def __init__(self, a: SemanticContext, b: SemanticContext) -> None: ...
    def __eq__(self, other): ...
    def __hash__(self): ...
    def eval(self, parser: Recognizer, outerContext: RuleContext): ...
    def evalPrecedence(self, parser: Recognizer, outerContext: RuleContext): ...

class OR(SemanticContext):
    opnds: Incomplete
    def __init__(self, a: SemanticContext, b: SemanticContext) -> None: ...
    def __eq__(self, other): ...
    def __hash__(self): ...
    def eval(self, parser: Recognizer, outerContext: RuleContext): ...
    def evalPrecedence(self, parser: Recognizer, outerContext: RuleContext): ...



CR_ERROR_FIRST: int
CR_UNKNOWN_ERROR: int
CR_SOCKET_CREATE_ERROR: int
CR_CONNECTION_ERROR: int
CR_CONN_HOST_ERROR: int
CR_IPSOCK_ERROR: int
CR_UNKNOWN_HOST: int
CR_SERVER_GONE_ERROR: int
CR_VERSION_ERROR: int
CR_OUT_OF_MEMORY: int
CR_WRONG_HOST_INFO: int
CR_LOCALHOST_CONNECTION: int
CR_TCP_CONNECTION: int
CR_SERVER_HANDSHAKE_ERR: int
CR_SERVER_LOST: int
CR_COMMANDS_OUT_OF_SYNC: int
CR_NAMEDPIPE_CONNECTION: int
CR_NAMEDPIPEWAIT_ERROR: int
CR_NAMEDPIPEOPEN_ERROR: int
CR_NAMEDPIPESETSTATE_ERROR: int
CR_CANT_READ_CHARSET: int
CR_NET_PACKET_TOO_LARGE: int
CR_EMBEDDED_CONNECTION: int
CR_PROBE_SLAVE_STATUS: int
CR_PROBE_SLAVE_HOSTS: int
CR_PROBE_SLAVE_CONNECT: int
CR_PROBE_MASTER_CONNECT: int
CR_SSL_CONNECTION_ERROR: int
CR_MALFORMED_PACKET: int
CR_WRONG_LICENSE: int
CR_NULL_POINTER: int
CR_NO_PREPARE_STMT: int
CR_PARAMS_NOT_BOUND: int
CR_DATA_TRUNCATED: int
CR_NO_PARAMETERS_EXISTS: int
CR_INVALID_PARAMETER_NO: int
CR_INVALID_BUFFER_USE: int
CR_UNSUPPORTED_PARAM_TYPE: int
CR_SHARED_MEMORY_CONNECTION: int
CR_SHARED_MEMORY_CONNECT_REQUEST_ERROR: int
CR_SHARED_MEMORY_CONNECT_ANSWER_ERROR: int
CR_SHARED_MEMORY_CONNECT_FILE_MAP_ERROR: int
CR_SHARED_MEMORY_CONNECT_MAP_ERROR: int
CR_SHARED_MEMORY_FILE_MAP_ERROR: int
CR_SHARED_MEMORY_MAP_ERROR: int
CR_SHARED_MEMORY_EVENT_ERROR: int
CR_SHARED_MEMORY_CONNECT_ABANDONED_ERROR: int
CR_SHARED_MEMORY_CONNECT_SET_ERROR: int
CR_CONN_UNKNOW_PROTOCOL: int
CR_INVALID_CONN_HANDLE: int
CR_SECURE_AUTH: int
CR_FETCH_CANCELED: int
CR_NO_DATA: int
CR_NO_STMT_METADATA: int
CR_NO_RESULT_SET: int
CR_NOT_IMPLEMENTED: int
CR_SERVER_LOST_EXTENDED: int
CR_STMT_CLOSED: int
CR_NEW_STMT_METADATA: int
CR_ALREADY_CONNECTED: int
CR_AUTH_PLUGIN_CANNOT_LOAD: int
CR_DUPLICATE_CONNECTION_ATTR: int
CR_AUTH_PLUGIN_ERR: int
CR_ERROR_LAST: int



from _typeshed import Incomplete

from ...extend.operation import ExtendedOperation
from ...protocol.novell import ReplicaList
from ...protocol.rfc4511 import LDAPDN

class ListReplicas(ExtendedOperation):
    request_name: str
    response_name: str
    request_value: LDAPDN
    asn1_spec: ReplicaList
    response_attribute: str
    def config(self) -> None: ...
    def __init__(self, connection, server_dn, controls: Incomplete | None = None) -> None: ...
    def populate_result(self) -> None: ...



import timeit
from _typeshed import Incomplete
from collections.abc import Generator
from hmac import compare_digest
from typing import Any

from passlib.utils.compat import JYTHON as JYTHON

__all__ = [
    "JYTHON",
    "sys_bits",
    "unix_crypt_schemes",
    "rounds_cost_values",
    "consteq",
    "saslprep",
    "xor_bytes",
    "render_bytes",
    "is_same_codec",
    "is_ascii_safe",
    "to_bytes",
    "to_unicode",
    "to_native_str",
    "has_crypt",
    "test_crypt",
    "safe_crypt",
    "tick",
    "rng",
    "getrandbytes",
    "getrandstr",
    "generate_password",
    "is_crypt_handler",
    "is_crypt_context",
    "has_rounds_info",
    "has_salt_info",
]

sys_bits: Any
unix_crypt_schemes: list[str]
rounds_cost_values: Any

class SequenceMixin:
    def __getitem__(self, idx): ...
    def __iter__(self): ...
    def __len__(self) -> int: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...

consteq = compare_digest

def str_consteq(left, right): ...
def saslprep(source, param: str = "value"): ...
def render_bytes(source, *args): ...
def xor_bytes(left, right): ...
def is_same_codec(left, right): ...
def is_ascii_safe(source): ...
def to_bytes(source, encoding: str = "utf-8", param: str = "value", source_encoding: Incomplete | None = None): ...
def to_unicode(source, encoding: str = "utf-8", param: str = "value"): ...
def to_native_str(source, encoding: str = "utf-8", param: str = "value"): ...

has_crypt: bool

def safe_crypt(secret, hash) -> None: ...
def test_crypt(secret, hash): ...

timer = timeit.default_timer
tick = timer
rng: Any

def getrandbytes(rng, count) -> Generator[None, None, Any]: ...
def getrandstr(rng, charset, count) -> Generator[None, None, Any]: ...
def generate_password(size: int = 10, charset=...): ...
def is_crypt_handler(obj): ...
def is_crypt_context(obj): ...
def has_rounds_info(handler): ...
def has_salt_info(handler): ...



from logging import Logger
from typing import Any, Literal

from ._types import Reader, TagDict

logger: Logger

class IfdTag:
    printable: str
    tag: int
    field_type: int
    field_offset: int
    field_length: int
    values: Any  # either string, bytes or list of data items
    def __init__(self, printable: str, tag: int, field_type: int, values: Any, field_offset: int, field_length: int) -> None: ...

class ExifHeader:
    file_handle: Reader
    endian: Literal["I", "M"]
    offset: int
    fake_exif: bool
    strict: bool
    debug: bool
    detailed: bool
    truncate_tags: bool
    tags: dict[str, Any]
    def __init__(
        self,
        file_handle: Reader,
        endian: Literal["I", "M"],
        offset: int,
        fake_exif: bool,
        strict: bool,
        debug: bool = False,
        detailed: bool = True,
        truncate_tags: bool = True,
    ) -> None: ...
    def s2n(self, offset: int, length: int, signed: bool = False) -> int: ...
    def n2b(self, offset: int, length: int) -> bytes: ...
    def list_ifd(self) -> list[int]: ...
    def dump_ifd(
        self, ifd: int, ifd_name: str, tag_dict: TagDict | None = None, relative: int = 0, stop_tag: str = "UNDEF"
    ) -> None: ...
    def extract_tiff_thumbnail(self, thumb_ifd: int) -> None: ...
    def extract_jpeg_thumbnail(self) -> None: ...
    def decode_maker_note(self) -> None: ...
    def parse_xmp(self, xmp_bytes: bytes) -> None: ...



from .noop_traceid import NoOpTraceId as NoOpTraceId
from .segment import Segment as Segment
from .subsegment import Subsegment as Subsegment
from .traceid import TraceId as TraceId

class DummySegment(Segment):
    sampled: bool
    def __init__(self, name: str = "dummy") -> None: ...
    def set_aws(self, aws_meta) -> None: ...
    def put_http_meta(self, key, value) -> None: ...
    def put_annotation(self, key, value) -> None: ...
    def put_metadata(self, key, value, namespace: str = "default") -> None: ...
    def set_user(self, user) -> None: ...
    def set_service(self, service_info) -> None: ...
    def apply_status_code(self, status_code) -> None: ...
    def add_exception(self, exception, stack, remote: bool = False) -> None: ...
    def serialize(self) -> None: ...

class DummySubsegment(Subsegment):
    sampled: bool
    def __init__(self, segment, name: str = "dummy") -> None: ...
    def set_aws(self, aws_meta) -> None: ...
    def put_http_meta(self, key, value) -> None: ...
    def put_annotation(self, key, value) -> None: ...
    def put_metadata(self, key, value, namespace: str = "default") -> None: ...
    def set_sql(self, sql) -> None: ...
    def apply_status_code(self, status_code) -> None: ...
    def add_exception(self, exception, stack, remote: bool = False) -> None: ...
    def serialize(self) -> None: ...



from _typeshed import Incomplete
from typing import Any

from . import base

fullTree: bool
tag_regexp: Any
comment_type: Any

class DocumentType:
    name: Any
    publicId: Any
    systemId: Any
    def __init__(self, name, publicId, systemId) -> None: ...

class Document:
    def __init__(self) -> None: ...
    def appendChild(self, element) -> None: ...
    @property
    def childNodes(self): ...

def testSerializer(element): ...
def tostring(element): ...

class TreeBuilder(base.TreeBuilder):
    documentClass: Any
    doctypeClass: Any
    elementClass: Any
    commentClass: Any
    fragmentClass: Any
    implementation: Any
    namespaceHTMLElements: Any
    def __init__(self, namespaceHTMLElements, fullTree: bool = False): ...
    insertComment: Any
    initial_comments: Any
    doctype: Any
    def reset(self) -> None: ...
    def testSerializer(self, element): ...
    def getDocument(self): ...
    def getFragment(self): ...
    def insertDoctype(self, token) -> None: ...
    def insertCommentInitial(self, data, parent: Incomplete | None = None) -> None: ...
    def insertCommentMain(self, data, parent: Incomplete | None = None) -> None: ...
    document: Any
    def insertRoot(self, token) -> None: ...



from _typeshed import Incomplete

HTTP_CONTINUE: int
HTTP_SWITCHING_PROTOCOLS: int
HTTP_PROCESSING: int
HTTP_OK: int
HTTP_CREATED: int
HTTP_ACCEPTED: int
HTTP_NON_AUTHORITATIVE: int
HTTP_NO_CONTENT: int
HTTP_RESET_CONTENT: int
HTTP_PARTIAL_CONTENT: int
HTTP_MULTI_STATUS: int
HTTP_MULTIPLE_CHOICES: int
HTTP_MOVED_PERMANENTLY: int
HTTP_MOVED_TEMPORARILY: int
HTTP_SEE_OTHER: int
HTTP_NOT_MODIFIED: int
HTTP_USE_PROXY: int
HTTP_TEMPORARY_REDIRECT: int
HTTP_BAD_REQUEST: int
HTTP_UNAUTHORIZED: int
HTTP_PAYMENT_REQUIRED: int
HTTP_FORBIDDEN: int
HTTP_NOT_FOUND: int
HTTP_METHOD_NOT_ALLOWED: int
HTTP_NOT_ACCEPTABLE: int
HTTP_PROXY_AUTHENTICATION_REQUIRED: int
HTTP_REQUEST_TIME_OUT: int
HTTP_CONFLICT: int
HTTP_GONE: int
HTTP_LENGTH_REQUIRED: int
HTTP_PRECONDITION_FAILED: int
HTTP_REQUEST_ENTITY_TOO_LARGE: int
HTTP_REQUEST_URI_TOO_LARGE: int
HTTP_UNSUPPORTED_MEDIA_TYPE: int
HTTP_RANGE_NOT_SATISFIABLE: int
HTTP_EXPECTATION_FAILED: int
HTTP_UNPROCESSABLE_ENTITY: int
HTTP_INTERNAL_SERVER_ERROR: int
HTTP_NOT_IMPLEMENTED: int
HTTP_BAD_GATEWAY: int
HTTP_SERVICE_UNAVAILABLE: int
HTTP_GATEWAY_TIME_OUT: int
HTTP_VERSION_NOT_SUPPORTED: int
HTTP_VARIANT_ALSO_VARIES: int
HSE_STATUS_SUCCESS: int
HSE_STATUS_SUCCESS_AND_KEEP_CONN: int
HSE_STATUS_PENDING: int
HSE_STATUS_ERROR: int
SF_NOTIFY_SECURE_PORT: int
SF_NOTIFY_NONSECURE_PORT: int
SF_NOTIFY_READ_RAW_DATA: int
SF_NOTIFY_PREPROC_HEADERS: int
SF_NOTIFY_AUTHENTICATION: int
SF_NOTIFY_URL_MAP: int
SF_NOTIFY_ACCESS_DENIED: int
SF_NOTIFY_SEND_RESPONSE: int
SF_NOTIFY_SEND_RAW_DATA: int
SF_NOTIFY_LOG: int
SF_NOTIFY_END_OF_REQUEST: int
SF_NOTIFY_END_OF_NET_SESSION: int
SF_NOTIFY_ORDER_HIGH: int
SF_NOTIFY_ORDER_MEDIUM: int
SF_NOTIFY_ORDER_LOW: int
SF_NOTIFY_ORDER_DEFAULT: int
SF_NOTIFY_ORDER_MASK: Incomplete
SF_STATUS_REQ_FINISHED: int
SF_STATUS_REQ_FINISHED_KEEP_CONN: Incomplete
SF_STATUS_REQ_NEXT_NOTIFICATION: Incomplete
SF_STATUS_REQ_HANDLED_NOTIFICATION: Incomplete
SF_STATUS_REQ_ERROR: Incomplete
SF_STATUS_REQ_READ_NEXT: Incomplete
HSE_IO_SYNC: int
HSE_IO_ASYNC: int
HSE_IO_DISCONNECT_AFTER_SEND: int
HSE_IO_SEND_HEADERS: int
HSE_IO_NODELAY: int
HSE_IO_FINAL_SEND: int
HSE_IO_CACHE_RESPONSE: int
HSE_EXEC_URL_NO_HEADERS: int
HSE_EXEC_URL_IGNORE_CURRENT_INTERCEPTOR: int
HSE_EXEC_URL_IGNORE_VALIDATION_AND_RANGE: int
HSE_EXEC_URL_DISABLE_CUSTOM_ERROR: int
HSE_EXEC_URL_SSI_CMD: int
HSE_EXEC_URL_HTTP_CACHE_ELIGIBLE: int



import sys
from collections.abc import Callable, Iterable
from ctypes import c_long
from typing import Literal, SupportsInt, TypeVar
from typing_extensions import TypeAlias

from ._generic import GenericListener as _GenericListener
from ._mouse_event import (
    DOUBLE as DOUBLE,
    DOWN as DOWN,
    LEFT as LEFT,
    MIDDLE as MIDDLE,
    RIGHT as RIGHT,
    UP as UP,
    X2 as X2,
    ButtonEvent as ButtonEvent,
    MoveEvent as MoveEvent,
    WheelEvent as WheelEvent,
    X as X,
    _MouseButton,
    _MouseEvent,
    _MouseEventType,
)

# mypy doesn't support PEP 646's TypeVarTuple yet: https://github.com/python/mypy/issues/12280
# _Ts = TypeVarTuple("_Ts")
_Ts: TypeAlias = tuple[object, ...]
_Callback: TypeAlias = Callable[[_MouseEvent], bool | None]
_C = TypeVar("_C", bound=_Callback)

class _MouseListener(_GenericListener):
    def init(self) -> None: ...
    def pre_process_event(  # type: ignore[override]  # Mouse specific events and return
        self, event: _MouseEvent
    ) -> Literal[True]: ...
    def listen(self) -> None: ...

def is_pressed(button: _MouseButton = "left") -> bool: ...
def press(button: _MouseButton = "left") -> None: ...
def release(button: _MouseButton = "left") -> None: ...
def click(button: _MouseButton = "left") -> None: ...
def double_click(button: _MouseButton = "left") -> None: ...
def right_click() -> None: ...
def wheel(delta: int = 1) -> None: ...
def move(x: SupportsInt, y: SupportsInt, absolute: bool = True, duration: float = 0) -> None: ...
def drag(start_x: int, start_y: int, end_x: int, end_y: int, absolute: bool = True, duration: float = 0) -> None: ...
def on_button(
    callback: Callable[..., None],
    args: _Ts = (),
    # Omitting default: Darwin has no x and x2
    buttons: list[_MouseButton] | tuple[_MouseButton, ...] | _MouseButton = ...,
    # Omitting default: Darwin and Linux don't have "double", yet the defaults includes it
    types: list[_MouseEventType] | tuple[_MouseEventType, ...] | _MouseEventType = ...,
) -> _Callback: ...
def on_click(callback: Callable[..., None], args: _Ts = ()) -> _Callback: ...
def on_double_click(callback: Callable[..., None], args: _Ts = ()) -> _Callback: ...
def on_right_click(callback: Callable[..., None], args: _Ts = ()) -> _Callback: ...
def on_middle_click(callback: Callable[..., None], args: _Ts = ()) -> _Callback: ...
def wait(
    button: _MouseButton = "left",
    # Omitting default: Darwin and Linux don't have "double", yet the defaults includes it
    target_types: tuple[_MouseEventType, ...] = ...,
) -> None: ...

if sys.platform == "win32":
    def get_position() -> tuple[c_long, c_long]: ...

else:
    def get_position() -> tuple[int, int]: ...

def hook(callback: _C) -> _C: ...
def unhook(callback: _Callback) -> None: ...
def unhook_all() -> None: ...
def record(button: _MouseButton = "right", target_types: tuple[_MouseEventType] = ("down",)) -> _MouseEvent: ...
def play(
    events: Iterable[_MouseEvent],
    speed_factor: float = 1.0,
    include_clicks: bool = True,
    include_moves: bool = True,
    include_wheel: bool = True,
) -> None: ...

replay = play
hold = press



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def attribute_mixing_dict(G, attribute, nodes: Incomplete | None = None, normalized: bool = False): ...
@_dispatch
def attribute_mixing_matrix(
    G, attribute, nodes: Incomplete | None = None, mapping: Incomplete | None = None, normalized: bool = True
): ...
@_dispatch
def degree_mixing_dict(
    G, x: str = "out", y: str = "in", weight: Incomplete | None = None, nodes: Incomplete | None = None, normalized: bool = False
): ...
@_dispatch
def degree_mixing_matrix(
    G,
    x: str = "out",
    y: str = "in",
    weight: Incomplete | None = None,
    nodes: Incomplete | None = None,
    normalized: bool = True,
    mapping: Incomplete | None = None,
): ...
@_dispatch
def mixing_dict(xy, normalized: bool = False): ...



from _typeshed import Incomplete

from influxdb_client.service._base_service import _BaseService

class WriteService(_BaseService):
    def __init__(self, api_client: Incomplete | None = None) -> None: ...
    def post_write(self, org, bucket, body, **kwargs): ...
    def post_write_with_http_info(self, org, bucket, body, **kwargs): ...
    async def post_write_async(self, org, bucket, body, **kwargs): ...



from _typeshed import Incomplete
from xml.sax import ErrorHandler as _ErrorHandler

from .expatreader import DefusedExpatParser

__origin__: str

def parse(
    source,
    handler,
    errorHandler: _ErrorHandler = ...,
    forbid_dtd: bool = False,
    forbid_entities: bool = True,
    forbid_external: bool = True,
) -> None: ...
def parseString(
    string,
    handler,
    errorHandler: _ErrorHandler = ...,
    forbid_dtd: bool = False,
    forbid_entities: bool = True,
    forbid_external: bool = True,
) -> None: ...
def make_parser(parser_list: list[Incomplete] = []) -> DefusedExpatParser: ...



from _typeshed import Incomplete

class PaginatedResult:
    total_items: Incomplete
    page_size: Incomplete
    current_page: Incomplete
    def __init__(self, total_items, page_size, current_page) -> None: ...



from _typeshed import Incomplete
from collections.abc import Generator

import qrcode.image.base

class PyPNGImage(qrcode.image.base.BaseImage):
    kind: str
    allowed_kinds: Incomplete
    needs_drawrect: bool
    def new_image(self, **kwargs): ...
    def drawrect(self, row, col) -> None: ...
    def save(self, stream, kind: Incomplete | None = None) -> None: ...
    def rows_iter(self) -> Generator[Incomplete, Incomplete, None]: ...
    def border_rows_iter(self) -> Generator[Incomplete, None, None]: ...

PymagingImage = PyPNGImage



from _typeshed import Incomplete
from typing import Any

change_table: Any

def modify_operation(
    dn, changes, auto_encode, schema: Incomplete | None = None, validator: Incomplete | None = None, check_names: bool = False
): ...
def modify_request_to_dict(request): ...
def modify_response_to_dict(response): ...



from _typeshed import Incomplete

class WebhookTestingGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def sample_notification(self, kind, id, source_merchant_id: Incomplete | None = None): ...



from _typeshed import Incomplete

def bind_operation(
    version,
    authentication,
    name: str = "",
    password: Incomplete | None = None,
    sasl_mechanism: Incomplete | None = None,
    sasl_credentials: Incomplete | None = None,
    auto_encode: bool = False,
): ...
def bind_request_to_dict(request): ...
def bind_response_operation(
    result_code,
    matched_dn: str = "",
    diagnostic_message: str = "",
    referral: Incomplete | None = None,
    server_sasl_credentials: Incomplete | None = None,
): ...
def bind_response_to_dict(response): ...
def sicily_bind_response_to_dict(response): ...
def bind_response_to_dict_fast(response): ...
def sicily_bind_response_to_dict_fast(response): ...



from _typeshed import Incomplete, StrPath
from typing import Generic, TypeVar

from .._distutils.dist import DistributionMetadata
from ..dist import Distribution
from . import expand

SingleCommandOptions: Incomplete
AllCommandOptions: Incomplete
Target = TypeVar("Target", bound=Distribution | DistributionMetadata)  # noqa: Y001 # Exists at runtime

def read_configuration(
    filepath: StrPath, find_others: bool = False, ignore_option_errors: bool = False
) -> dict[Incomplete, Incomplete]: ...
def apply_configuration(dist: Distribution, filepath: StrPath) -> Distribution: ...
def configuration_to_dict(
    handlers: tuple[ConfigHandler[Distribution | DistributionMetadata], ...]
) -> dict[Incomplete, Incomplete]: ...
def parse_configuration(
    distribution: Distribution, command_options: AllCommandOptions, ignore_option_errors: bool = False
) -> tuple[ConfigMetadataHandler, ConfigOptionsHandler]: ...

class ConfigHandler(Generic[Target]):
    section_prefix: str
    aliases: dict[str, str]
    ignore_option_errors: Incomplete
    target_obj: Incomplete
    sections: Incomplete
    set_options: Incomplete
    ensure_discovered: Incomplete
    def __init__(
        self,
        target_obj: Target,
        options: AllCommandOptions,
        ignore_option_errors,
        ensure_discovered: expand.EnsurePackagesDiscovered,
    ) -> None: ...
    @property
    def parsers(self) -> None: ...
    def __setitem__(self, option_name, value): ...
    def parse_section(self, section_options) -> None: ...
    def parse(self) -> None: ...

class ConfigMetadataHandler(ConfigHandler[DistributionMetadata]):
    section_prefix: str
    aliases: Incomplete
    strict_mode: bool
    package_dir: Incomplete
    root_dir: Incomplete
    def __init__(
        self,
        target_obj: DistributionMetadata,
        options: AllCommandOptions,
        ignore_option_errors: bool,
        ensure_discovered: expand.EnsurePackagesDiscovered,
        package_dir: dict[Incomplete, Incomplete] | None = None,
        root_dir: StrPath = ".",
    ) -> None: ...
    @property
    def parsers(self): ...

class ConfigOptionsHandler(ConfigHandler[Distribution]):
    section_prefix: str
    root_dir: Incomplete
    package_dir: Incomplete
    def __init__(
        self,
        target_obj: Distribution,
        options: AllCommandOptions,
        ignore_option_errors: bool,
        ensure_discovered: expand.EnsurePackagesDiscovered,
    ) -> None: ...
    @property
    def parsers(self): ...
    def parse_section_packages__find(self, section_options): ...
    def parse_section_entry_points(self, section_options) -> None: ...
    def parse_section_package_data(self, section_options) -> None: ...
    def parse_section_exclude_package_data(self, section_options) -> None: ...
    def parse_section_extras_require(self, section_options): ...
    def parse_section_data_files(self, section_options) -> None: ...



class GeneratedProtocolMessageType(type):
    def __new__(cls, name, bases, dictionary): ...
    def __init__(self, /, name, bases, dictionary) -> None: ...

def ParseMessage(descriptor, byte_str): ...
def MakeClass(descriptor): ...



from _typeshed import Incomplete
from typing import NamedTuple

from psutil._common import (
    NIC_DUPLEX_FULL as NIC_DUPLEX_FULL,
    NIC_DUPLEX_HALF as NIC_DUPLEX_HALF,
    NIC_DUPLEX_UNKNOWN as NIC_DUPLEX_UNKNOWN,
    AccessDenied as AccessDenied,
    NoSuchProcess as NoSuchProcess,
    ZombieProcess as ZombieProcess,
    conn_to_ntuple as conn_to_ntuple,
    get_procfs_path as get_procfs_path,
    memoize_when_activated as memoize_when_activated,
    usage_percent as usage_percent,
)
from psutil._compat import (
    PY3 as PY3,
    FileNotFoundError as FileNotFoundError,
    PermissionError as PermissionError,
    ProcessLookupError as ProcessLookupError,
)

__extra__all__: Incomplete
HAS_THREADS: Incomplete
HAS_NET_IO_COUNTERS: Incomplete
HAS_PROC_IO_COUNTERS: Incomplete
PAGE_SIZE: Incomplete
AF_LINK: Incomplete
PROC_STATUSES: Incomplete
TCP_STATUSES: Incomplete
proc_info_map: Incomplete

class pmem(NamedTuple):
    rss: Incomplete
    vms: Incomplete

pfullmem = pmem

class scputimes(NamedTuple):
    user: Incomplete
    system: Incomplete
    idle: Incomplete
    iowait: Incomplete

class svmem(NamedTuple):
    total: Incomplete
    available: Incomplete
    percent: Incomplete
    used: Incomplete
    free: Incomplete

def virtual_memory(): ...
def swap_memory(): ...
def cpu_times(): ...
def per_cpu_times(): ...
def cpu_count_logical(): ...
def cpu_count_cores(): ...
def cpu_stats(): ...

disk_io_counters: Incomplete
disk_usage: Incomplete

def disk_partitions(all: bool = ...): ...

net_if_addrs: Incomplete
net_io_counters: Incomplete

def net_connections(kind, _pid: int = ...): ...
def net_if_stats(): ...
def boot_time(): ...
def users(): ...
def pids(): ...
def pid_exists(pid): ...
def wrap_exceptions(fun): ...

class Process:
    pid: Incomplete
    def __init__(self, pid) -> None: ...
    def oneshot_enter(self) -> None: ...
    def oneshot_exit(self) -> None: ...
    def name(self): ...
    def exe(self): ...
    def cmdline(self): ...
    def environ(self): ...
    def create_time(self): ...
    def num_threads(self): ...
    def threads(self): ...
    def connections(self, kind: str = ...): ...
    def nice_get(self): ...
    def nice_set(self, value): ...
    def ppid(self): ...
    def uids(self): ...
    def gids(self): ...
    def cpu_times(self): ...
    def terminal(self): ...
    def cwd(self): ...
    def memory_info(self): ...
    memory_full_info: Incomplete
    def status(self): ...
    def open_files(self): ...
    def num_fds(self): ...
    def num_ctx_switches(self): ...
    def wait(self, timeout: Incomplete | None = ...): ...
    def io_counters(self): ...



from _typeshed import Incomplete

import arrow

class DateMathException(Exception): ...

def parse(
    expression: str, now: arrow.Arrow | None = None, tz: str = "UTC", type: str | None = None, roundDown: bool = True
) -> arrow.Arrow: ...
def __getattr__(name: str) -> Incomplete: ...



from networkx.algorithms.approximation.clique import *
from networkx.algorithms.approximation.clustering_coefficient import *
from networkx.algorithms.approximation.connectivity import *
from networkx.algorithms.approximation.distance_measures import *
from networkx.algorithms.approximation.dominating_set import *
from networkx.algorithms.approximation.kcomponents import *
from networkx.algorithms.approximation.matching import *
from networkx.algorithms.approximation.maxcut import *
from networkx.algorithms.approximation.ramsey import *
from networkx.algorithms.approximation.steinertree import *
from networkx.algorithms.approximation.traveling_salesman import *
from networkx.algorithms.approximation.treewidth import *
from networkx.algorithms.approximation.vertex_cover import *



import sys
from _typeshed import FileDescriptor
from collections.abc import Callable
from types import TracebackType
from typing import Protocol
from typing_extensions import TypeAlias, TypeVarTuple, Unpack

from gevent._types import _AsyncWatcher, _Callback, _ChildWatcher, _IoWatcher, _StatWatcher, _TimerWatcher, _Watcher

_Ts = TypeVarTuple("_Ts")
_ErrorHandlerFunc: TypeAlias = Callable[
    [object | None, type[BaseException] | None, BaseException | None, TracebackType | None], object
]

class _SupportsHandleError(Protocol):
    handle_error: _ErrorHandlerFunc

_ErrorHandler: TypeAlias = _ErrorHandlerFunc | _SupportsHandleError

class AbstractLoop:
    CALLBACK_CHECK_COUNT: int
    error_handler: _ErrorHandler | None
    starting_timer_may_update_loop_time: bool
    # internal API, this __init__ will only be called from subclasses
    def __init__(
        self, ffi: object, lib: object, watchers: object, flags: int | None = ..., default: bool | None = ...
    ) -> None: ...
    def destroy(self) -> bool | None: ...
    @property
    def ptr(self) -> int: ...
    @property
    def WatcherType(self) -> type[_Watcher]: ...
    @property
    def MAXPRI(self) -> int: ...
    @property
    def MINPRI(self) -> int: ...
    def handle_error(
        self, context: object | None, type: type[BaseException] | None, value: BaseException | None, tb: TracebackType | None
    ) -> None: ...
    def run(self, nowait: bool = False, once: bool = False) -> None: ...
    def reinit(self) -> None: ...
    def ref(self) -> None: ...
    def unref(self) -> None: ...
    def break_(self, how: int | None = ...) -> None: ...
    def verify(self) -> None: ...
    def now(self) -> float: ...
    def update_now(self) -> None: ...
    update = update_now  # deprecated
    @property
    def default(self) -> bool: ...
    @property
    def iteration(self) -> int: ...
    @property
    def depth(self) -> int: ...
    @property
    def backend_int(self) -> int: ...
    @property
    def backend(self) -> str | int: ...
    @property
    def pendingcnt(self) -> int: ...
    @property
    def activecnt(self) -> int: ...
    def io(self, fd: FileDescriptor, events: int, ref: bool = True, priority: int | None = None) -> _IoWatcher: ...
    def closing_fd(self, fd: FileDescriptor) -> bool: ...
    def timer(self, after: float, repeat: float = 0.0, ref: bool = True, priority: int | None = None) -> _TimerWatcher: ...
    def signal(self, signum: int, ref: bool = True, priority: int | None = None) -> _Watcher: ...
    def idle(self, ref: bool = True, priority: int | None = None) -> _Watcher: ...
    def prepare(self, ref: bool = True, priority: int | None = None) -> _Watcher: ...
    def check(self, ref: bool = True, priority: int | None = None) -> _Watcher: ...
    if sys.platform != "win32":
        def fork(self, ref: bool = True, priority: int | None = None) -> _Watcher: ...
        def child(self, pid: int, trace: int = 0, ref: bool = True) -> _ChildWatcher: ...
        def install_sigchld(self) -> None: ...

    def async_(self, ref: bool = True, priority: int | None = None) -> _AsyncWatcher: ...
    def stat(self, path: str, interval: float = 0.0, ref: bool = True, priority: bool | None = ...) -> _StatWatcher: ...
    def run_callback(self, func: Callable[[Unpack[_Ts]], object], *args: Unpack[_Ts]) -> _Callback: ...
    def run_callback_threadsafe(self, func: Callable[[Unpack[_Ts]], object], *args: Unpack[_Ts]) -> _Callback: ...
    def callback(self, priority: float | None = ...) -> _Callback: ...
    def fileno(self) -> FileDescriptor | None: ...



from _typeshed import Incomplete
from collections.abc import Iterator, Sequence
from typing import Any
from xml.etree.ElementTree import Element, ElementTree, ParseError as ParseError, XMLParser as _XMLParser, tostring as tostring

class DefusedXMLParser(_XMLParser):
    forbid_dtd: bool
    forbid_entities: bool
    forbid_external: bool
    def __init__(
        self,
        html=...,
        target: Incomplete | None = None,
        encoding: str | None = None,
        forbid_dtd: bool = False,
        forbid_entities: bool = True,
        forbid_external: bool = True,
    ) -> None: ...
    def defused_start_doctype_decl(self, name, sysid, pubid, has_internal_subset) -> None: ...
    def defused_entity_decl(self, name, is_parameter_entity, value, base, sysid, pubid, notation_name) -> None: ...
    def defused_unparsed_entity_decl(self, name, base, sysid, pubid, notation_name) -> None: ...
    def defused_external_entity_ref_handler(self, context, base, sysid, pubid) -> None: ...

XMLTreeBuilder = DefusedXMLParser
XMLParse = DefusedXMLParser
XMLParser = DefusedXMLParser

# wrapper to xml.etree.ElementTree.parse
def parse(
    source, parser: XMLParser | None = None, forbid_dtd: bool = False, forbid_entities: bool = True, forbid_external: bool = True
) -> ElementTree: ...

# wrapper to xml.etree.ElementTree.iterparse
def iterparse(
    source,
    events: Sequence[str] | None = None,
    parser: XMLParser | None = None,
    forbid_dtd: bool = False,
    forbid_entities: bool = True,
    forbid_external: bool = True,
) -> Iterator[tuple[str, Any]]: ...
def fromstring(text, forbid_dtd: bool = False, forbid_entities: bool = True, forbid_external: bool = True) -> Element: ...

XML = fromstring

__all__ = ["ParseError", "XML", "XMLParse", "XMLParser", "XMLTreeBuilder", "fromstring", "iterparse", "parse", "tostring"]



class PyAsn1Error(Exception):
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def context(self): ...

class ValueConstraintError(PyAsn1Error): ...
class SubstrateUnderrunError(PyAsn1Error): ...
class EndOfStreamError(SubstrateUnderrunError): ...
class UnsupportedSubstrateError(PyAsn1Error): ...

class PyAsn1UnicodeError(PyAsn1Error, UnicodeError):
    def __init__(self, message, unicode_error: UnicodeError | None = None) -> None: ...

class PyAsn1UnicodeDecodeError(PyAsn1UnicodeError, UnicodeDecodeError): ...
class PyAsn1UnicodeEncodeError(PyAsn1UnicodeError, UnicodeEncodeError): ...



import sys
from typing import Literal, NamedTuple
from typing_extensions import TypeAlias

_MouseEvent: TypeAlias = ButtonEvent | WheelEvent | MoveEvent  # noqa: Y047  # Used outside

LEFT: Literal["left"]
RIGHT: Literal["right"]
MIDDLE: Literal["middle"]
X: Literal["x"]
X2: Literal["x2"]

UP: Literal["up"]
DOWN: Literal["down"]
DOUBLE: Literal["double"]
WHEEL: Literal["wheel"]

VERTICAL: Literal["vertical"]
HORIZONTAL: Literal["horizontal"]

if sys.platform == "linux" or sys.platform == "win32":
    _MouseButton: TypeAlias = Literal["left", "right", "middle", "x", "x2"]
else:
    _MouseButton: TypeAlias = Literal["left", "right", "middle"]

if sys.platform == "win32":
    _MouseEventType: TypeAlias = Literal["up", "down", "double", "wheel"]
else:
    _MouseEventType: TypeAlias = Literal["up", "down"]

class ButtonEvent(NamedTuple):
    event_type: _MouseEventType
    button: _MouseButton
    time: float

class WheelEvent(NamedTuple):
    delta: int
    time: float

class MoveEvent(NamedTuple):
    x: int
    y: int
    time: float



from pytz import BaseTzInfo
from tzlocal.utils import assert_tz_offset

__all__ = ["get_localzone", "get_localzone_name", "reload_localzone", "assert_tz_offset"]

def reload_localzone() -> None: ...
def get_localzone() -> BaseTzInfo: ...
def get_localzone_name() -> str: ...



from _typeshed import Incomplete

class TelegrafPlugins:
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(
        self, version: Incomplete | None = None, os: Incomplete | None = None, plugins: Incomplete | None = None
    ) -> None: ...
    @property
    def version(self): ...
    @version.setter
    def version(self, version) -> None: ...
    @property
    def os(self): ...
    @os.setter
    def os(self, os) -> None: ...
    @property
    def plugins(self): ...
    @plugins.setter
    def plugins(self, plugins) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from typing import ClassVar

from .core import UnitedStates

class Illinois(UnitedStates):
    include_thanksgiving_friday: ClassVar[bool]
    include_lincoln_birthday: ClassVar[bool]
    include_election_day_even: ClassVar[bool]

class ChicagoIllinois(Illinois):
    include_thanksgiving_friday: ClassVar[bool]
    def get_pulaski_day(self, year): ...
    def get_variable_days(self, year): ...



from _typeshed import Incomplete

class Label:
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(
        self,
        id: Incomplete | None = None,
        org_id: Incomplete | None = None,
        name: Incomplete | None = None,
        properties: Incomplete | None = None,
    ) -> None: ...
    @property
    def id(self): ...
    @id.setter
    def id(self, id) -> None: ...
    @property
    def org_id(self): ...
    @org_id.setter
    def org_id(self, org_id) -> None: ...
    @property
    def name(self): ...
    @name.setter
    def name(self, name) -> None: ...
    @property
    def properties(self): ...
    @properties.setter
    def properties(self, properties) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from _typeshed import Incomplete, Unused

from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.xml.functions import Element

class Related(Serialisable):
    id: Incomplete
    def __init__(self, id: Incomplete | None = None) -> None: ...
    def to_tree(self, tagname: str | None, idx: Unused = None) -> Element: ...  # type: ignore[override]



from typing import Final

from Xlib._typing import ErrorHandler
from Xlib.display import Display
from Xlib.protocol import request, rq
from Xlib.xobject import drawable

extname: Final = "MIT-SCREEN-SAVER"
NotifyMask: Final = 1
CycleMask: Final = 2
StateOff: Final = 0
StateOn: Final = 1
StateCycle: Final = 2
KindBlanked: Final = 0
KindInternal: Final = 1
KindExternal: Final = 2

class QueryVersion(rq.ReplyRequest): ...

def query_version(self: drawable.Drawable) -> QueryVersion: ...

class QueryInfo(rq.ReplyRequest): ...

def query_info(self: drawable.Drawable) -> QueryInfo: ...

class SelectInput(rq.Request): ...

def select_input(self: drawable.Drawable, mask: int) -> SelectInput: ...

class SetAttributes(rq.Request): ...

def set_attributes(
    self: drawable.Drawable,
    x: int,
    y: int,
    width: int,
    height: int,
    border_width: int,
    window_class: int = 0,
    depth: int = 0,
    visual: int = 0,
    onerror: ErrorHandler[object] | None = None,
    **keys: object,
) -> SetAttributes: ...

class UnsetAttributes(rq.Request): ...

def unset_attributes(self: drawable.Drawable, onerror: ErrorHandler[object] | None = None) -> UnsetAttributes: ...

class Notify(rq.Event): ...

def init(disp: Display, info: request.QueryExtension) -> None: ...



from typing import Any

from pynput import _util

from ._base import Button as Button, Controller as Controller, Listener as Listener

class Events(_util.Events[Any, Listener]):
    class Move(_util.Events.Event):
        x: int
        y: int
        def __init__(self, x: int, y: int) -> None: ...

    class Click(_util.Events.Event):
        x: int
        y: int
        button: Button
        pressed: bool
        def __init__(self, x: int, y: int, button: Button, pressed: bool) -> None: ...

    class Scroll(_util.Events.Event):
        x: int
        y: int
        dx: int
        dy: int
        def __init__(self, x: int, y: int, dx: int, dy: int) -> None: ...

    def __init__(self) -> None: ...
    def __next__(self) -> Move | Click | Scroll: ...
    def get(self, timeout: float | None = None) -> Move | Click | Scroll | None: ...



from _typeshed import Unused
from typing import Final

from Xlib.display import Display
from Xlib.protocol import rq
from Xlib.xobject import resource

extname: Final = "Generic Event Extension"
GenericEventCode: Final = 35

class GEQueryVersion(rq.ReplyRequest): ...

def query_version(self: Display | resource.Resource) -> GEQueryVersion: ...

class GenericEvent(rq.Event): ...

def add_event_data(self: Display | resource.Resource, extension: int, evtype: int, estruct: int) -> None: ...
def init(disp: Display, info: Unused) -> None: ...



from _typeshed import Incomplete
from dataclasses import dataclass
from typing import Any, Literal
from typing_extensions import TypeAlias

from fpdf.enums import Align
from fpdf.fpdf import FPDF

from .image_parsing import _ImageFilter

_AlignLiteral: TypeAlias = Literal[
    "",
    "CENTER",
    "X_CENTER",
    "LEFT",
    "RIGHT",
    "JUSTIFY",
    "center",
    "x_center",
    "left",
    "right",
    "justify",
    "C",
    "X",
    "L",
    "R",
    "J",
    "c",
    "x",
    "l",
    "r",
    "j",
]

class ImageInfo(dict[str, Any]):
    @property
    def width(self) -> int: ...
    @property
    def height(self) -> int: ...
    @property
    def rendered_width(self) -> int: ...
    @property
    def rendered_height(self) -> int: ...
    def scale_inside_box(self, x: float, y: float, w: float, h: float) -> tuple[float, float, float, float]: ...
    @staticmethod
    def x_by_align(x: Align | _AlignLiteral, w: float, pdf: FPDF, keep_aspect_ratio: Literal[False]) -> float: ...

class RasterImageInfo(ImageInfo):
    def size_in_document_units(self, w: float, h: float, scale=1) -> tuple[float, float]: ...

class VectorImageInfo(ImageInfo): ...

@dataclass
class ImageCache:
    images: dict[str, dict[Incomplete, Incomplete]] = ...
    icc_profiles: dict[bytes, int] = ...
    image_filter: _ImageFilter = "AUTO"

    def reset_usages(self) -> None: ...



from _typeshed import Incomplete
from collections.abc import Generator

from networkx.algorithms.flow import edmonds_karp
from networkx.utils.backends import _dispatch

__all__ = ["all_node_cuts"]

default_flow_func = edmonds_karp

@_dispatch
def all_node_cuts(G, k: Incomplete | None = None, flow_func: Incomplete | None = None) -> Generator[Incomplete, None, None]: ...



from _typeshed import Incomplete

from ..core import Calendar

class Japan(Calendar):
    WEEKEND_DAYS: Incomplete
    FIXED_HOLIDAYS: Incomplete
    def get_fixed_holidays(self, year): ...
    def get_variable_days(self, year): ...

class JapanBank(Japan):
    FIXED_HOLIDAYS: Incomplete



import logging
from _typeshed import Incomplete
from typing import Any

from .s3.connection import S3Connection

Version: Any
UserAgent: Any
config: Any
BUCKET_NAME_RE: Any
TOO_LONG_DNS_NAME_COMP: Any
GENERATION_RE: Any
VERSION_RE: Any
ENDPOINTS_PATH: Any

def init_logging(): ...

class NullHandler(logging.Handler):
    def emit(self, record): ...

log: Any
perflog: Any

def set_file_logger(name, filepath, level: Any = 20, format_string: Incomplete | None = None): ...
def set_stream_logger(name, level: Any = 10, format_string: Incomplete | None = None): ...
def connect_sqs(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_s3(aws_access_key_id: str | None = None, aws_secret_access_key: str | None = None, **kwargs) -> S3Connection: ...
def connect_gs(gs_access_key_id: Incomplete | None = None, gs_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_ec2(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_elb(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_autoscale(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_cloudwatch(
    aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs
): ...
def connect_sdb(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_fps(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_mturk(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_cloudfront(
    aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs
): ...
def connect_vpc(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_rds(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_rds2(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_emr(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_sns(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_iam(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_route53(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_cloudformation(
    aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs
): ...
def connect_euca(
    host: Incomplete | None = None,
    aws_access_key_id: Incomplete | None = None,
    aws_secret_access_key: Incomplete | None = None,
    port: int = 8773,
    path: str = "/services/Eucalyptus",
    is_secure: bool = False,
    **kwargs,
): ...
def connect_glacier(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_ec2_endpoint(
    url, aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs
): ...
def connect_walrus(
    host: Incomplete | None = None,
    aws_access_key_id: Incomplete | None = None,
    aws_secret_access_key: Incomplete | None = None,
    port: int = 8773,
    path: str = "/services/Walrus",
    is_secure: bool = False,
    **kwargs,
): ...
def connect_ses(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_sts(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_ia(
    ia_access_key_id: Incomplete | None = None, ia_secret_access_key: Incomplete | None = None, is_secure: bool = False, **kwargs
): ...
def connect_dynamodb(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_swf(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_cloudsearch(
    aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs
): ...
def connect_cloudsearch2(
    aws_access_key_id: Incomplete | None = None,
    aws_secret_access_key: Incomplete | None = None,
    sign_request: bool = False,
    **kwargs,
): ...
def connect_cloudsearchdomain(
    aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs
): ...
def connect_beanstalk(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_elastictranscoder(
    aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs
): ...
def connect_opsworks(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_redshift(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_support(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_cloudtrail(
    aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs
): ...
def connect_directconnect(
    aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs
): ...
def connect_kinesis(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_logs(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_route53domains(
    aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs
): ...
def connect_cognito_identity(
    aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs
): ...
def connect_cognito_sync(
    aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs
): ...
def connect_kms(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_awslambda(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_codedeploy(
    aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs
): ...
def connect_configservice(
    aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs
): ...
def connect_cloudhsm(aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs): ...
def connect_ec2containerservice(
    aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs
): ...
def connect_machinelearning(
    aws_access_key_id: Incomplete | None = None, aws_secret_access_key: Incomplete | None = None, **kwargs
): ...
def storage_uri(
    uri_str,
    default_scheme: str = "file",
    debug: int = 0,
    validate: bool = True,
    bucket_storage_uri_class: Any = ...,
    suppress_consec_slashes: bool = True,
    is_latest: bool = False,
): ...
def storage_uri_for_key(key): ...

# Explicitly mark this package as incomplete.
def __getattr__(name: str) -> Incomplete: ...



from stripe.api_resources.abstract import (
    CreateableAPIResource as CreateableAPIResource,
    ListableAPIResource as ListableAPIResource,
    UpdateableAPIResource as UpdateableAPIResource,
    custom_method as custom_method,
)

class SubscriptionSchedule(CreateableAPIResource, ListableAPIResource, UpdateableAPIResource):
    OBJECT_NAME: str
    def cancel(self, idempotency_key: str | None = None, **params): ...
    def release(self, idempotency_key: str | None = None, **params): ...



from _typeshed import Incomplete
from collections.abc import Generator

from antlr4.tree.Tree import INVALID_INTERVAL as INVALID_INTERVAL, ParseTreeVisitor as ParseTreeVisitor, RuleNode as RuleNode
from antlr4.tree.Trees import Trees as Trees

Parser: Incomplete

class RuleContext(RuleNode):
    EMPTY: Incomplete
    parentCtx: Incomplete
    invokingState: Incomplete
    def __init__(self, parent: RuleContext | None = None, invokingState: int = -1) -> None: ...
    def depth(self): ...
    def isEmpty(self): ...
    def getSourceInterval(self): ...
    def getRuleContext(self): ...
    def getPayload(self): ...
    def getText(self): ...
    def getRuleIndex(self): ...
    def getAltNumber(self): ...
    def setAltNumber(self, altNumber: int): ...
    def getChild(self, i: int): ...
    def getChildCount(self): ...
    def getChildren(self) -> Generator[Incomplete, None, None]: ...
    def accept(self, visitor: ParseTreeVisitor): ...
    def toStringTree(self, ruleNames: list[Incomplete] | None = None, recog: Parser | None = None): ...
    def toString(self, ruleNames: list[Incomplete], stop: RuleContext) -> str: ...



from typing import ClassVar

import passlib.utils.handlers as uh

class des_crypt(uh.TruncateMixin, uh.HasManyBackends, uh.HasSalt, uh.GenericHandler):  # type: ignore[misc]
    name: ClassVar[str]
    checksum_chars: ClassVar[str]
    checksum_size: ClassVar[int]
    min_salt_size: ClassVar[int]
    max_salt_size: ClassVar[int]
    salt_chars: ClassVar[str]
    truncate_size: ClassVar[int]
    @classmethod
    def from_string(cls, hash): ...
    backends: ClassVar[tuple[str, ...]]

class bsdi_crypt(uh.HasManyBackends, uh.HasRounds, uh.HasSalt, uh.GenericHandler):  # type: ignore[misc]
    name: ClassVar[str]
    checksum_size: ClassVar[int]
    checksum_chars: ClassVar[str]
    min_salt_size: ClassVar[int]
    max_salt_size: ClassVar[int]
    salt_chars: ClassVar[str]
    default_rounds: ClassVar[int]
    min_rounds: ClassVar[int]
    max_rounds: ClassVar[int]
    rounds_cost: ClassVar[str]
    @classmethod
    def from_string(cls, hash): ...
    @classmethod
    def using(cls, **kwds): ...
    backends: ClassVar[tuple[str, ...]]

class bigcrypt(uh.HasSalt, uh.GenericHandler):
    name: ClassVar[str]
    checksum_chars: ClassVar[str]
    min_salt_size: ClassVar[int]
    max_salt_size: ClassVar[int]
    salt_chars: ClassVar[str]
    @classmethod
    def from_string(cls, hash): ...

class crypt16(uh.TruncateMixin, uh.HasSalt, uh.GenericHandler):  # type: ignore[misc]
    name: ClassVar[str]
    checksum_size: ClassVar[int]
    checksum_chars: ClassVar[str]
    min_salt_size: ClassVar[int]
    max_salt_size: ClassVar[int]
    salt_chars: ClassVar[str]
    truncate_size: ClassVar[int]
    @classmethod
    def from_string(cls, hash): ...



from collections.abc import Callable, MutableMapping
from typing import Any, ClassVar, NoReturn, TypedDict

from jmespath.functions import Functions

class Options:
    dict_cls: Callable[[], MutableMapping[Any, Any]] | None
    custom_functions: Functions | None
    def __init__(
        self, dict_cls: Callable[[], MutableMapping[Any, Any]] | None = None, custom_functions: Functions | None = None
    ) -> None: ...

class _Expression:
    expression: str
    interpreter: Visitor
    def __init__(self, expression: str, interpreter: Visitor) -> None: ...
    def visit(self, node: _TreeNode, *args, **kwargs) -> Any: ...

class Visitor:
    def __init__(self) -> None: ...
    def visit(self, node: _TreeNode, *args, **kwargs) -> Any: ...
    def default_visit(self, node: _TreeNode, *args, **kwargs) -> NoReturn: ...

class _TreeNode(TypedDict):
    type: str
    value: Any
    children: list[_TreeNode]

class TreeInterpreter(Visitor):
    COMPARATOR_FUNC: ClassVar[dict[str, Callable[[Any, Any], Any]]]
    MAP_TYPE: ClassVar[Callable[[], MutableMapping[Any, Any]]]
    def __init__(self, options: Options | None = None) -> None: ...
    def default_visit(self, node: _TreeNode, *args, **kwargs) -> NoReturn: ...
    def visit_subexpression(self, node: _TreeNode, value: Any) -> Any: ...
    def visit_field(self, node: _TreeNode, value: Any) -> Any: ...
    def visit_comparator(self, node: _TreeNode, value: Any) -> Any: ...
    def visit_current(self, node: _TreeNode, value: Any) -> Any: ...
    def visit_expref(self, node: _TreeNode, value: Any) -> Any: ...
    def visit_function_expression(self, node: _TreeNode, value: Any) -> Any: ...
    def visit_filter_projection(self, node: _TreeNode, value: Any) -> Any: ...
    def visit_flatten(self, node: _TreeNode, value: Any) -> Any: ...
    def visit_identity(self, node: _TreeNode, value: Any) -> Any: ...
    def visit_index(self, node: _TreeNode, value: Any) -> Any: ...
    def visit_index_expression(self, node: _TreeNode, value: Any) -> Any: ...
    def visit_slice(self, node: _TreeNode, value: Any) -> Any: ...
    def visit_key_val_pair(self, node: _TreeNode, value: Any) -> Any: ...
    def visit_literal(self, node: _TreeNode, value: Any) -> Any: ...
    def visit_multi_select_dict(self, node: _TreeNode, value: Any) -> Any: ...
    def visit_multi_select_list(self, node: _TreeNode, value: Any) -> Any: ...
    def visit_or_expression(self, node: _TreeNode, value: Any) -> Any: ...
    def visit_and_expression(self, node: _TreeNode, value: Any) -> Any: ...
    def visit_not_expression(self, node: _TreeNode, value: Any) -> Any: ...
    def visit_pipe(self, node: _TreeNode, value: Any) -> Any: ...
    def visit_projection(self, node: _TreeNode, value: Any) -> Any: ...
    def visit_value_projection(self, node: _TreeNode, value: Any) -> Any: ...

class GraphvizVisitor(Visitor):
    def __init__(self) -> None: ...
    def visit(self, node: _TreeNode, *args, **kwargs) -> str: ...



from collections.abc import Iterable, Iterator, Sequence
from re import Pattern
from typing import Any

from pyflakes.reporter import Reporter

__all__ = ["check", "checkPath", "checkRecursive", "iterSourceCode", "main"]

PYTHON_SHEBANG_REGEX: Pattern[bytes]

def check(codeString: str, filename: str, reporter: Reporter | None = None) -> int: ...
def checkPath(filename, reporter: Reporter | None = None) -> int: ...
def isPythonFile(filename) -> bool: ...
def iterSourceCode(paths: Iterable[Any]) -> Iterator[Any]: ...
def checkRecursive(paths: Iterable[Any], reporter: Reporter) -> int: ...
def main(prog: str | None = None, args: Sequence[Any] | None = None) -> None: ...



from _typeshed import StrOrBytesPath
from collections.abc import Iterator
from typing import IO, Any

from webob.dec import wsgify
from webob.request import Request
from webob.response import Response

BLOCK_SIZE: int

class FileApp:
    filename: StrOrBytesPath
    kw: dict[str, Any]
    def __init__(self, filename: StrOrBytesPath, **kw: Any) -> None: ...
    @wsgify
    def __call__(self, req: Request) -> Response: ...

class FileIter:
    file: IO[bytes]
    def __init__(self, file: IO[bytes]) -> None: ...
    def app_iter_range(
        self, seek: int | None = None, limit: int | None = None, block_size: int | None = None
    ) -> Iterator[bytes]: ...
    __iter__ = app_iter_range

class DirectoryApp:
    path: str | bytes
    index_page: str | None
    hide_index_with_redirect: bool
    fileapp_kw: dict[str, Any]
    def __init__(
        self, path: StrOrBytesPath, index_page: str = "index.html", hide_index_with_redirect: bool = False, **kw: Any
    ) -> None: ...
    def make_fileapp(self, path: StrOrBytesPath) -> FileApp: ...
    @wsgify
    def __call__(self, req: Request) -> Response | FileApp: ...
    def index(self, req: Request, path: StrOrBytesPath) -> Response | FileApp: ...



from _typeshed import Incomplete
from collections.abc import Iterable
from typing import Protocol
from typing_extensions import TypeAlias

from .Image import Image, Resampling, _Resample, _Size
from .ImageColor import _Ink

_Border: TypeAlias = int | tuple[int, int] | tuple[int, int, int, int]

class _Deformer(Protocol):
    def getmesh(self, image: Image, /): ...

def autocontrast(
    image: Image,
    cutoff: float | tuple[float, float] = 0,
    ignore: int | None = None,
    mask: Image | None = None,
    preserve_tone: bool = False,
) -> Image: ...
def colorize(
    image: Image,
    black: int | str,
    white: int | str,
    mid: int | str | None = None,
    blackpoint: int = 0,
    whitepoint: int = 255,
    midpoint: int = 127,
) -> Image: ...
def contain(image: Image, size: _Size, method: Resampling | _Resample = ...) -> Image: ...
def cover(image: Image, size: _Size, method: Resampling | _Resample = ...) -> Image: ...
def pad(
    image: Image,
    size: _Size,
    method: Resampling | _Resample = ...,
    color: Incomplete | None = None,
    centering: Iterable[float] = (0.5, 0.5),
) -> Image: ...
def crop(image: Image, border: _Border = 0) -> Image: ...
def scale(image: Image, factor: float, resample: Resampling | _Resample = ...) -> Image: ...
def deform(image: Image, deformer: _Deformer, resample: Resampling | _Resample = ...) -> Image: ...
def equalize(image: Image, mask: Incomplete | None = None) -> Image: ...
def expand(image: Image, border: _Border = 0, fill: _Ink = 0) -> Image: ...
def fit(
    image: Image, size: _Size, method: Resampling | _Resample = ..., bleed: float = 0.0, centering: Iterable[float] = (0.5, 0.5)
) -> Image: ...
def flip(image: Image) -> Image: ...
def grayscale(image: Image) -> Image: ...
def invert(image: Image) -> Image: ...
def mirror(image: Image) -> Image: ...
def posterize(image: Image, bits: int) -> Image: ...
def solarize(image: Image, threshold: int = 128) -> Image: ...
def exif_transpose(image: Image, *, in_place: bool = False) -> Image: ...



from collections.abc import Mapping, Sequence
from typing import Any
from xml.etree.ElementTree import Element

from markdown import inlinepatterns, util
from markdown.core import Markdown
from markdown.extensions import Extension
from markdown.inlinepatterns import HtmlInlineProcessor

punctClass: str
endOfWordClass: str
closeClass: str
openingQuotesBase: str
substitutions: Mapping[str, str]
singleQuoteStartRe: str
doubleQuoteStartRe: str
doubleQuoteSetsRe: str
singleQuoteSetsRe: str
decadeAbbrRe: str
openingDoubleQuotesRegex: str
closingDoubleQuotesRegex: str
closingDoubleQuotesRegex2: str
openingSingleQuotesRegex: str
closingSingleQuotesRegex: str
closingSingleQuotesRegex2: str
remainingSingleQuotesRegex: str
remainingDoubleQuotesRegex: str
HTML_STRICT_RE: str

class SubstituteTextPattern(HtmlInlineProcessor):
    replace: Sequence[int | str | Element]
    def __init__(self, pattern: str, replace: Sequence[int | str | Element], md: Markdown) -> None: ...

class SmartyExtension(Extension):
    substitutions: Any
    def __init__(self, **kwargs) -> None: ...
    def educateDashes(self, md: Markdown) -> None: ...
    def educateEllipses(self, md: Markdown) -> None: ...
    def educateAngledQuotes(self, md: Markdown) -> None: ...
    def educateQuotes(self, md: Markdown) -> None: ...
    inlinePatterns: util.Registry[inlinepatterns.Pattern]

def makeExtension(**kwargs) -> SmartyExtension: ...



import sys
import xml.dom.minidom
from _typeshed import Incomplete
from abc import abstractmethod
from collections import Counter
from collections.abc import Callable, Generator, Iterable, Iterator, Mapping, Sequence
from typing import Any, ClassVar, Literal, Protocol, SupportsIndex, TypeVar, overload
from typing_extensions import Self, TypeAlias

from docutils.frontend import Values
from docutils.transforms import Transform, Transformer
from docutils.utils import Reporter

_N = TypeVar("_N", bound=Node)

class _DomModule(Protocol):
    Document: type[xml.dom.minidom.Document]

# Functional Node Base Classes

class Node:
    # children is initialized by the subclasses
    children: Sequence[Node]
    # TODO: `parent` is actually `Element | None``, but `None`` only happens rarely,
    #       i.e. for synthetic nodes (or `document`, where it is overridden).
    #       See https://github.com/python/typeshed/blob/main/CONTRIBUTING.md#the-any-trick
    parent: Element | Any
    source: str | None
    line: int | None
    document: document | None
    def __bool__(self) -> Literal[True]: ...
    def asdom(self, dom: _DomModule | None = None) -> xml.dom.minidom.Element: ...
    # While docutils documents the Node class to be abstract it does not
    # actually use the ABCMeta metaclass. We still set @abstractmethod here
    # (although it's not used in the docutils implementation) because it
    # makes Mypy reject Node() with "Cannot instantiate abstract class".
    @abstractmethod
    def copy(self) -> Self: ...
    @abstractmethod
    def deepcopy(self) -> Self: ...
    @abstractmethod
    def pformat(self, indent: str = "    ", level: int = 0) -> str: ...
    @abstractmethod
    def astext(self) -> str: ...
    def setup_child(self, child: Node) -> None: ...
    def walk(self, visitor: NodeVisitor) -> bool: ...
    def walkabout(self, visitor: NodeVisitor) -> bool: ...
    @overload
    def findall(
        self, condition: type[_N], include_self: bool = True, descend: bool = True, siblings: bool = False, ascend: bool = False
    ) -> Generator[_N, None, None]: ...
    @overload
    def findall(
        self,
        condition: Callable[[Node], bool] | None = None,
        include_self: bool = True,
        descend: bool = True,
        siblings: bool = False,
        ascend: bool = False,
    ) -> Generator[Node, None, None]: ...
    @overload
    def traverse(
        self, condition: type[_N], include_self: bool = True, descend: bool = True, siblings: bool = False, ascend: bool = False
    ) -> list[_N]: ...
    @overload
    def traverse(
        self,
        condition: Callable[[Node], bool] | None = None,
        include_self: bool = True,
        descend: bool = True,
        siblings: bool = False,
        ascend: bool = False,
    ) -> list[Node]: ...
    @overload
    def next_node(
        self, condition: type[_N], include_self: bool = False, descend: bool = True, siblings: bool = False, ascend: bool = False
    ) -> _N: ...
    @overload
    def next_node(
        self,
        condition: Callable[[Node], bool] | None = None,
        include_self: bool = False,
        descend: bool = True,
        siblings: bool = False,
        ascend: bool = False,
    ) -> Node: ...

# Left out
# - def ensure_str (deprecated)
# - def unescape (canonical import from docutils.utils)

class Text(Node, str):
    tagname: ClassVar[str]
    children: tuple[()]

    # we omit the rawsource parameter because it has been deprecated and is ignored
    def __new__(cls, data: str) -> Self: ...
    def __init__(self, data: str) -> None: ...
    def shortrepr(self, maxlen: int = 18) -> str: ...
    def copy(self) -> Self: ...
    def deepcopy(self) -> Self: ...
    def pformat(self, indent: str = "    ", level: int = 0) -> str: ...
    def astext(self) -> str: ...
    def rstrip(self, chars: str | None = None) -> str: ...
    def lstrip(self, chars: str | None = None) -> str: ...

_T = TypeVar("_T")

class Element(Node):
    local_attributes: ClassVar[Sequence[str]]
    basic_attributes: ClassVar[Sequence[str]]
    list_attributes: ClassVar[Sequence[str]]
    known_attributes: ClassVar[Sequence[str]]
    tagname: str
    child_text_separator: ClassVar[str]
    attributes: dict[str, Any]
    children: list[Node]
    rawsource: str
    def __init__(self, rawsource: str = "", *children: Node, **attributes: Any) -> None: ...
    def shortrepr(self) -> str: ...
    def starttag(self, quoteattr: Callable[[str], str] | None = None) -> str: ...
    def endtag(self) -> str: ...
    def emptytag(self) -> str: ...
    def __len__(self) -> int: ...
    def __contains__(self, key: str | Node) -> bool: ...
    @overload
    def __getitem__(self, key: str) -> Any: ...
    @overload
    def __getitem__(self, key: int) -> Node: ...
    @overload
    def __getitem__(self, key: slice) -> list[Node]: ...
    @overload
    def __setitem__(self, key: str, item: Any) -> None: ...
    @overload
    def __setitem__(self, key: int, item: Node) -> None: ...
    @overload
    def __setitem__(self, key: slice, item: Iterable[Node]) -> None: ...
    def __delitem__(self, key: str | int | slice) -> None: ...
    def __add__(self, other: list[Node]) -> list[Node]: ...
    def __radd__(self, other: list[Node]) -> list[Node]: ...
    def __iadd__(self, other: Node | Iterable[Node]) -> Self: ...
    def astext(self) -> str: ...
    def non_default_attributes(self) -> dict[str, Any]: ...
    def attlist(self) -> list[tuple[str, Any]]: ...
    @overload
    def get(self, key: str) -> Any: ...
    @overload
    def get(self, key: str, failobj: _T) -> _T: ...
    def hasattr(self, attr: str) -> bool: ...
    def delattr(self, attr: str) -> None: ...
    @overload
    def setdefault(self, key: str) -> Any: ...
    @overload
    def setdefault(self, key: str, failobj: _T) -> Any | _T: ...
    has_key = hasattr
    def get_language_code(self, fallback: str = "") -> str: ...
    def append(self, item: Node) -> None: ...
    def extend(self, item: Iterable[Node]) -> None: ...
    def insert(self, index: SupportsIndex, item: Node | Iterable[Node] | None) -> None: ...
    def pop(self, i: int = -1) -> Node: ...
    def remove(self, item: Node) -> None: ...
    def index(self, item: Node, start: int = 0, stop: int = sys.maxsize) -> int: ...
    def previous_sibling(self) -> Node | None: ...
    def is_not_default(self, key: str) -> bool: ...
    def update_basic_atts(self, dict_: Mapping[str, Any] | Node) -> None: ...
    def append_attr_list(self, attr: str, values: Iterable[Any]) -> None: ...
    def coerce_append_attr_list(self, attr: str, value) -> None: ...
    def replace_attr(self, attr: str, value: Any, force: bool = True) -> None: ...
    def copy_attr_convert(self, attr: str, value: Any, replace: bool = True) -> None: ...
    def copy_attr_coerce(self, attr: str, value: Any, replace: bool) -> None: ...
    def copy_attr_concatenate(self, attr: str, value: Any, replace: bool) -> None: ...
    def copy_attr_consistent(self, attr: str, value: Any, replace: bool) -> None: ...
    def update_all_atts(
        self,
        dict_: Mapping[str, Any] | Node,
        update_fun: Callable[[Element, str, Any, bool], object] = ...,
        replace: bool = True,
        and_source: bool = False,
    ) -> None: ...
    def update_all_atts_consistantly(
        self, dict_: Mapping[str, Any] | Node, replace: bool = True, and_source: bool = False
    ) -> None: ...
    def update_all_atts_concatenating(
        self, dict_: dict[str, Any] | Node, replace: bool = True, and_source: bool = False
    ) -> None: ...
    def update_all_atts_coercion(
        self, dict_: Mapping[str, Any] | Node, replace: bool = True, and_source: bool = False
    ) -> None: ...
    def update_all_atts_convert(self, dict_: Mapping[str, Any] | Node, and_source: bool = False) -> None: ...
    def clear(self) -> None: ...
    def replace(self, old: Node, new: Node | Sequence[Node]) -> None: ...
    def replace_self(self, new: Node | Sequence[Node]) -> None: ...
    def first_child_matching_class(
        self, childclass: type[Node] | tuple[type[Node], ...], start: int = 0, end: int = sys.maxsize
    ) -> int | None: ...
    def first_child_not_matching_class(
        self, childclass: type[Node] | tuple[type[Node], ...], start: int = 0, end: int = sys.maxsize
    ) -> int | None: ...
    def pformat(self, indent: str = "    ", level: int = 0) -> str: ...
    def copy(self) -> Self: ...
    def deepcopy(self) -> Self: ...
    def set_class(self, name: str) -> None: ...
    def note_referenced_by(self, name: str | None = None, id: str | None = None) -> None: ...
    @classmethod
    def is_not_list_attribute(cls, attr: str) -> bool: ...
    @classmethod
    def is_not_known_attribute(cls, attr: str) -> bool: ...

    # '__iter__' is added as workaround, since mypy doesn't support classes that are iterable via '__getitem__'
    # see https://github.com/python/typeshed/pull/10099#issuecomment-1528789395
    def __iter__(self) -> Iterator[Node]: ...

class TextElement(Element):
    def __init__(self, rawsource: str = "", text: str = "", *children: Node, **attributes) -> None: ...

class FixedTextElement(TextElement): ...

# Mixins

class Resolvable:
    resolved: int

class BackLinkable:
    def add_backref(self, refid: str) -> None: ...

# Element Categories

class Root: ...
class Titular: ...
class PreBibliographic: ...
class Bibliographic: ...
class Decorative(PreBibliographic): ...
class Structural: ...
class Body: ...
class General(Body): ...
class Sequential(Body): ...
class Admonition(Body): ...
class Special(Body): ...
class Invisible(PreBibliographic): ...
class Part: ...
class Inline: ...
class Referential(Resolvable): ...

class Targetable(Resolvable):
    referenced: int
    indirect_reference_name: str | None

class Labeled: ...

# Root Element

_Document: TypeAlias = document
_Decoration: TypeAlias = decoration

class document(Root, Structural, Element):
    current_source: str | None
    current_line: int | None
    settings: Values
    reporter: Reporter
    indirect_targets: list[target]
    substitution_defs: dict[str, substitution_definition]
    substitution_names: dict[str, str]
    refnames: dict[str, list[Element]]
    refids: dict[str, list[Element]]
    nameids: dict[str, str]
    nametypes: dict[str, bool]
    ids: dict[str, Element]
    footnote_refs: dict[str, list[footnote_reference]]
    citation_refs: dict[str, list[citation_reference]]
    autofootnotes: list[footnote]
    autofootnote_refs: list[footnote_reference]
    symbol_footnotes: list[footnote]
    symbol_footnote_refs: list[footnote_reference]
    footnotes: list[footnote]
    citations: list[citation]
    autofootnote_start: int
    symbol_footnote_start: int
    id_counter: Counter[int]
    parse_messages: list[system_message]
    transform_messages: list[system_message]
    transformer: Transformer
    decoration: decoration | None
    document: Self
    def __init__(self, settings: Values, reporter: Reporter, *args: Node, **kwargs: Any) -> None: ...
    def asdom(self, dom: Any | None = None) -> Any: ...
    def set_id(self, node: Element, msgnode: Element | None = None, suggested_prefix: str = "") -> str: ...
    def set_name_id_map(self, node: Element, id: str, msgnode: Element | None = None, explicit: bool | None = None) -> None: ...
    def set_duplicate_name_id(self, node: Element, id: str, name: str, msgnode: Element, explicit: bool) -> None: ...
    def has_name(self, name: str) -> bool: ...
    def note_implicit_target(self, target: Element, msgnode: Element | None = None) -> None: ...
    def note_explicit_target(self, target: Element, msgnode: Element | None = None) -> None: ...
    def note_refname(self, node: Element) -> None: ...
    def note_refid(self, node: Element) -> None: ...
    def note_indirect_target(self, target: target) -> None: ...
    def note_anonymous_target(self, target: target) -> None: ...
    def note_autofootnote(self, footnote: footnote) -> None: ...
    def note_autofootnote_ref(self, ref: footnote_reference) -> None: ...
    def note_symbol_footnote(self, footnote: footnote) -> None: ...
    def note_symbol_footnote_ref(self, ref: footnote_reference) -> None: ...
    def note_footnote(self, footnote: footnote) -> None: ...
    def note_footnote_ref(self, ref: footnote_reference) -> None: ...
    def note_citation(self, citation: citation) -> None: ...
    def note_citation_ref(self, ref: citation_reference) -> None: ...
    def note_substitution_def(self, subdef: substitution_definition, def_name: str, msgnode: Element | None = None) -> None: ...
    def note_substitution_ref(self, subref: substitution_reference, refname: str) -> None: ...
    def note_pending(self, pending: pending, priority: int | None = None) -> None: ...
    def note_parse_message(self, message: system_message) -> None: ...
    def note_transform_message(self, message: system_message) -> None: ...
    def note_source(self, source: str, offset: int) -> None: ...
    def copy(self) -> Self: ...
    def get_decoration(self) -> _Decoration: ...

# Title Elements

class title(Titular, PreBibliographic, TextElement): ...
class subtitle(Titular, PreBibliographic, TextElement): ...
class rubric(Titular, TextElement): ...

# Meta-Data Element

class meta(PreBibliographic, Element): ...

# Bibliographic Elements

class docinfo(Bibliographic, Element): ...
class author(Bibliographic, TextElement): ...
class authors(Bibliographic, Element): ...
class organization(Bibliographic, TextElement): ...
class address(Bibliographic, FixedTextElement): ...
class contact(Bibliographic, TextElement): ...
class version(Bibliographic, TextElement): ...
class revision(Bibliographic, TextElement): ...
class status(Bibliographic, TextElement): ...
class date(Bibliographic, TextElement): ...
class copyright(Bibliographic, TextElement): ...

# Decorative Elements

class decoration(Decorative, Element):
    def get_header(self) -> header: ...
    def get_footer(self) -> footer: ...

class header(Decorative, Element): ...
class footer(Decorative, Element): ...

# Structural Elements

class section(Structural, Element): ...
class topic(Structural, Element): ...
class sidebar(Structural, Element): ...
class transition(Structural, Element): ...

# Body Elements
# ===============

class paragraph(General, TextElement): ...
class compound(General, Element): ...
class container(General, Element): ...
class bullet_list(Sequential, Element): ...
class enumerated_list(Sequential, Element): ...
class list_item(Part, Element): ...
class definition_list(Sequential, Element): ...
class definition_list_item(Part, Element): ...
class term(Part, TextElement): ...
class classifier(Part, TextElement): ...
class definition(Part, Element): ...
class field_list(Sequential, Element): ...
class field(Part, Element): ...
class field_name(Part, TextElement): ...
class field_body(Part, Element): ...
class option(Part, Element): ...
class option_argument(Part, TextElement): ...
class option_group(Part, Element): ...
class option_list(Sequential, Element): ...
class option_list_item(Part, Element): ...
class option_string(Part, TextElement): ...
class description(Part, Element): ...
class literal_block(General, FixedTextElement): ...
class doctest_block(General, FixedTextElement): ...
class math_block(General, FixedTextElement): ...
class line_block(General, Element): ...

class line(Part, TextElement):
    indent: str | None

class block_quote(General, Element): ...
class attribution(Part, TextElement): ...
class attention(Admonition, Element): ...
class caution(Admonition, Element): ...
class danger(Admonition, Element): ...
class error(Admonition, Element): ...
class important(Admonition, Element): ...
class note(Admonition, Element): ...
class tip(Admonition, Element): ...
class hint(Admonition, Element): ...
class warning(Admonition, Element): ...
class admonition(Admonition, Element): ...
class comment(Special, Invisible, FixedTextElement): ...
class substitution_definition(Special, Invisible, TextElement): ...
class target(Special, Invisible, Inline, TextElement, Targetable): ...
class footnote(General, BackLinkable, Element, Labeled, Targetable): ...
class citation(General, BackLinkable, Element, Labeled, Targetable): ...
class label(Part, TextElement): ...
class figure(General, Element): ...
class caption(Part, TextElement): ...
class legend(Part, Element): ...
class table(General, Element): ...
class tgroup(Part, Element): ...
class colspec(Part, Element): ...
class thead(Part, Element): ...
class tbody(Part, Element): ...
class row(Part, Element): ...
class entry(Part, Element): ...

class system_message(Special, BackLinkable, PreBibliographic, Element):
    def __init__(self, message: str | None = None, *children: Node, **attributes) -> None: ...

class pending(Special, Invisible, Element):
    transform: Transform
    details: Mapping[str, Any]
    def __init__(
        self, transform: Transform, details: Mapping[str, Any] | None = None, rawsource: str = "", *children: Node, **attributes
    ) -> None: ...

class raw(Special, Inline, PreBibliographic, FixedTextElement): ...

# Inline Elements

class emphasis(Inline, TextElement): ...
class strong(Inline, TextElement): ...
class literal(Inline, TextElement): ...
class reference(General, Inline, Referential, TextElement): ...
class footnote_reference(Inline, Referential, TextElement): ...
class citation_reference(Inline, Referential, TextElement): ...
class substitution_reference(Inline, TextElement): ...
class title_reference(Inline, TextElement): ...
class abbreviation(Inline, TextElement): ...
class acronym(Inline, TextElement): ...
class superscript(Inline, TextElement): ...
class subscript(Inline, TextElement): ...
class math(Inline, TextElement): ...
class image(General, Inline, Element): ...
class inline(Inline, TextElement): ...
class problematic(Inline, TextElement): ...
class generated(Inline, TextElement): ...

# Auxiliary Classes, Functions, and Data

node_class_names: list[str]

class NodeVisitor:
    optional: ClassVar[tuple[str, ...]]
    document: _Document
    def __init__(self, document: _Document) -> None: ...
    def dispatch_visit(self, node: Node) -> Any: ...
    def dispatch_departure(self, node: Node) -> Any: ...
    def unknown_visit(self, node: Node) -> Any: ...
    def unknown_departure(self, node: Node) -> Any: ...

class SparseNodeVisitor(NodeVisitor): ...

class GenericNodeVisitor(NodeVisitor):
    # all the visit_<node_class_name> methods
    def __getattr__(self, name: str, /) -> Incomplete: ...

class TreeCopyVisitor(GenericNodeVisitor):
    parent_stack: list[Node]
    parent: list[Node]
    def get_tree_copy(self) -> Node: ...

class TreePruningException(Exception): ...
class SkipChildren(TreePruningException): ...
class SkipSiblings(TreePruningException): ...
class SkipNode(TreePruningException): ...
class SkipDeparture(TreePruningException): ...
class NodeFound(TreePruningException): ...
class StopTraversal(TreePruningException): ...

def make_id(string: str) -> str: ...
def dupname(node: Node, name: str) -> None: ...
def fully_normalize_name(name: str) -> str: ...
def whitespace_normalize_name(name: str) -> str: ...
def serial_escape(value: str) -> str: ...
def pseudo_quoteattr(value: str) -> str: ...



from collections.abc import Callable
from re import Match
from typing import Any, Literal

class JSONDecoder:
    encoding: str
    object_hook: Callable[[dict[Any, Any]], Any] | None
    parse_float: Callable[[str], Any] | None
    parse_int: Callable[[str], Any] | None
    parse_constant: Callable[[str], Any] | None
    strict: bool
    object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None
    memo: dict[Any, Any]
    scan_once: Callable[[str, int], tuple[bool, int]]

    def __init__(
        self,
        encoding: str | None = ...,
        object_hook: Callable[[dict[Any, Any]], Any] | None = ...,
        parse_float: Callable[[str], Any] | None = ...,
        parse_int: Callable[[str], Any] | None = ...,
        parse_constant: Callable[[str], Any] | None = ...,
        strict: bool = ...,
        object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None = ...,
        allow_nan: bool = ...,
    ) -> None: ...
    def decode(self, s: str, _w: Callable[[str, int], Match[str]] = ..., _PY3: Literal[True] = ...) -> Any: ...
    def raw_decode(
        self, s: str, idx: int = ..., _w: Callable[[str, int], Match[str]] = ..., _PY3: Literal[True] = ...
    ) -> tuple[Any, int]: ...



from _typeshed import Incomplete
from typing import Any

__author__: str
__copyright__: str
__license__: str

class FlexTemplate:
    pdf: Any
    splitting_pdf: Any
    handlers: Any
    texts: Any
    def __init__(self, pdf, elements: Incomplete | None = None) -> None: ...
    elements: Any
    keys: Any
    def load_elements(self, elements) -> None: ...
    def parse_csv(self, infile, delimiter: str = ",", decimal_sep: str = ".", encoding: Incomplete | None = None): ...
    def __setitem__(self, name, value) -> None: ...
    set: Any
    def __contains__(self, name): ...
    def __getitem__(self, name): ...
    def split_multicell(self, text, element_name): ...
    def render(self, offsetx: float = 0.0, offsety: float = 0.0, rotate: float = 0.0, scale: float = 1.0): ...

class Template(FlexTemplate):
    def __init__(
        self,
        infile: Incomplete | None = None,
        elements: Incomplete | None = None,
        format: str = "A4",
        orientation: str = "portrait",
        unit: str = "mm",
        title: str = "",
        author: str = "",
        subject: str = "",
        creator: str = "",
        keywords: str = "",
    ) -> None: ...
    def add_page(self) -> None: ...
    def render(self, outfile: Incomplete | None = None, dest: Incomplete | None = None) -> None: ...  # type: ignore[override]



from types import ModuleType
from typing import Any

class DeprecatableModule(ModuleType):
    def __init__(self, module: ModuleType) -> None: ...
    def __getattribute__(self, name: str) -> Any: ...

def deprecate_module_member(mod_name: str, name: str, message: str) -> None: ...



from typing import Final

XK_emspace: Final = 0xAA1
XK_enspace: Final = 0xAA2
XK_em3space: Final = 0xAA3
XK_em4space: Final = 0xAA4
XK_digitspace: Final = 0xAA5
XK_punctspace: Final = 0xAA6
XK_thinspace: Final = 0xAA7
XK_hairspace: Final = 0xAA8
XK_emdash: Final = 0xAA9
XK_endash: Final = 0xAAA
XK_signifblank: Final = 0xAAC
XK_ellipsis: Final = 0xAAE
XK_doubbaselinedot: Final = 0xAAF
XK_onethird: Final = 0xAB0
XK_twothirds: Final = 0xAB1
XK_onefifth: Final = 0xAB2
XK_twofifths: Final = 0xAB3
XK_threefifths: Final = 0xAB4
XK_fourfifths: Final = 0xAB5
XK_onesixth: Final = 0xAB6
XK_fivesixths: Final = 0xAB7
XK_careof: Final = 0xAB8
XK_figdash: Final = 0xABB
XK_leftanglebracket: Final = 0xABC
XK_decimalpoint: Final = 0xABD
XK_rightanglebracket: Final = 0xABE
XK_marker: Final = 0xABF
XK_oneeighth: Final = 0xAC3
XK_threeeighths: Final = 0xAC4
XK_fiveeighths: Final = 0xAC5
XK_seveneighths: Final = 0xAC6
XK_trademark: Final = 0xAC9
XK_signaturemark: Final = 0xACA
XK_trademarkincircle: Final = 0xACB
XK_leftopentriangle: Final = 0xACC
XK_rightopentriangle: Final = 0xACD
XK_emopencircle: Final = 0xACE
XK_emopenrectangle: Final = 0xACF
XK_leftsinglequotemark: Final = 0xAD0
XK_rightsinglequotemark: Final = 0xAD1
XK_leftdoublequotemark: Final = 0xAD2
XK_rightdoublequotemark: Final = 0xAD3
XK_prescription: Final = 0xAD4
XK_minutes: Final = 0xAD6
XK_seconds: Final = 0xAD7
XK_latincross: Final = 0xAD9
XK_hexagram: Final = 0xADA
XK_filledrectbullet: Final = 0xADB
XK_filledlefttribullet: Final = 0xADC
XK_filledrighttribullet: Final = 0xADD
XK_emfilledcircle: Final = 0xADE
XK_emfilledrect: Final = 0xADF
XK_enopencircbullet: Final = 0xAE0
XK_enopensquarebullet: Final = 0xAE1
XK_openrectbullet: Final = 0xAE2
XK_opentribulletup: Final = 0xAE3
XK_opentribulletdown: Final = 0xAE4
XK_openstar: Final = 0xAE5
XK_enfilledcircbullet: Final = 0xAE6
XK_enfilledsqbullet: Final = 0xAE7
XK_filledtribulletup: Final = 0xAE8
XK_filledtribulletdown: Final = 0xAE9
XK_leftpointer: Final = 0xAEA
XK_rightpointer: Final = 0xAEB
XK_club: Final = 0xAEC
XK_diamond: Final = 0xAED
XK_heart: Final = 0xAEE
XK_maltesecross: Final = 0xAF0
XK_dagger: Final = 0xAF1
XK_doubledagger: Final = 0xAF2
XK_checkmark: Final = 0xAF3
XK_ballotcross: Final = 0xAF4
XK_musicalsharp: Final = 0xAF5
XK_musicalflat: Final = 0xAF6
XK_malesymbol: Final = 0xAF7
XK_femalesymbol: Final = 0xAF8
XK_telephone: Final = 0xAF9
XK_telephonerecorder: Final = 0xAFA
XK_phonographcopyright: Final = 0xAFB
XK_caret: Final = 0xAFC
XK_singlelowquotemark: Final = 0xAFD
XK_doublelowquotemark: Final = 0xAFE
XK_cursor: Final = 0xAFF



from _typeshed import ConvertibleToInt, SliceableBuffer, Unused
from array import array

# Avoid name collision with List.type
from builtins import type as Type
from collections.abc import Callable, Iterable, Sequence
from typing import Any, Final, Literal, SupportsIndex, TypeVar, overload, type_check_only
from typing_extensions import LiteralString, TypeAlias

from Xlib._typing import ErrorHandler
from Xlib.display import _BaseDisplay, _ResourceBaseClass
from Xlib.error import XError
from Xlib.ext.xinput import ClassInfoClass
from Xlib.protocol import display

_T = TypeVar("_T")
_ModifierMappingList8Elements: TypeAlias = Sequence[Sequence[int]]

def decode_string(bs: bytes | bytearray) -> str: ...
def encode_array(a: array[Any] | memoryview) -> str: ...

class BadDataError(Exception): ...

signed_codes: Final[dict[int, str]]
unsigned_codes: Final[dict[int, str]]
array_unsigned_codes: Final[dict[int, LiteralString]]
struct_to_array_codes: Final[dict[str, LiteralString]]

class Field:
    name: str
    default: int | None
    pack_value: Callable[[Any], tuple[Any, int | None, int | None]] | None
    structcode: str | None
    structvalues: int
    check_value: Callable[[Any], Any] | None
    parse_value: Callable[[Any, Any], Any] | None
    keyword_args: int

    def parse_binary_value(
        self, data: SliceableBuffer, display: display.Display | None, length: int | None, format: int
    ) -> tuple[Any, SliceableBuffer]: ...

class Pad(Field):
    size: int
    value: bytes
    structcode: str
    def __init__(self, size: int) -> None: ...

class ConstantField(Field):
    value: int
    def __init__(self, value: int) -> None: ...

class Opcode(ConstantField):
    structcode: str

class ReplyCode(ConstantField):
    structcode: str
    value: int
    def __init__(self) -> None: ...

class LengthField(Field):
    structcode: str
    other_fields: list[str] | tuple[str, ...] | None
    def calc_length(self, length: int) -> int: ...

class TotalLengthField(LengthField): ...
class RequestLength(TotalLengthField): ...
class ReplyLength(TotalLengthField): ...

class LengthOf(LengthField):
    other_fields: list[str] | tuple[str, ...] | None
    def __init__(self, name: str | list[str] | tuple[str, ...], size: int) -> None: ...

class OddLength(LengthField):
    def __init__(self, name: str) -> None: ...
    def parse_value(self, value: int, display: Unused) -> Literal["even", "odd"]: ...  # type: ignore[override]

class FormatField(Field):
    structcode: str
    def __init__(self, name: str, size: int) -> None: ...

Format = FormatField

class ValueField(Field):
    def __init__(self, name: str, default: int | None = None) -> None: ...

class Int8(ValueField):
    structcode: str

class Int16(ValueField):
    structcode: str

class Int32(ValueField):
    structcode: str

class Card8(ValueField):
    structcode: str

class Card16(ValueField):
    structcode: str

class Card32(ValueField):
    structcode: str

class Resource(Card32):
    cast_function: str
    class_name: str
    codes: tuple[int, ...]
    def __init__(self, name: str, codes: tuple[int, ...] = (), default: int | None = None) -> None: ...
    @overload  # type: ignore[override]
    def check_value(self, value: Callable[[], _T]) -> _T: ...
    @overload
    def check_value(self, value: _T) -> _T: ...
    def parse_value(self, value: int, display: _BaseDisplay) -> int: ...  # type: ignore[override]  # display: None will error. See: https://github.com/python-xlib/python-xlib/pull/248

class Window(Resource):
    cast_function: str
    class_name: str

class Pixmap(Resource):
    cast_function: str
    class_name: str

class Drawable(Resource):
    cast_function: str
    class_name: str

class Fontable(Resource):
    cast_function: str
    class_name: str

class Font(Resource):
    cast_function: str
    class_name: str

class GC(Resource):
    cast_function: str
    class_name: str

class Colormap(Resource):
    cast_function: str
    class_name: str

class Cursor(Resource):
    cast_function: str
    class_name: str

class Bool(ValueField):
    structcode: str
    def check_value(self, value: object) -> bool: ...  # type: ignore[override]

class Set(ValueField):
    structcode: str
    values: Sequence[object]
    def __init__(self, name: str, size: int, values: Sequence[object], default: int | None = None) -> None: ...
    def check_value(self, val: _T) -> _T: ...  # type: ignore[override]

class Gravity(Set):
    def __init__(self, name: str) -> None: ...

class FixedBinary(ValueField):
    structcode: str
    def __init__(self, name: str, size: int) -> None: ...

class Binary(ValueField):
    structcode: None
    pad: int
    def __init__(self, name: str, pad: int = 1) -> None: ...
    def pack_value(  # type: ignore[override]  # Override Callable
        self, val: bytes | bytearray
    ) -> tuple[bytes | bytearray, int, None]: ...
    @overload  # type: ignore[override]  # Overload for specific values
    def parse_binary_value(self, data: _T, display: Unused, length: None, format: Unused) -> tuple[_T, Literal[b""]]: ...
    @overload
    def parse_binary_value(
        self, data: SliceableBuffer, display: Unused, length: int, format: Unused
    ) -> tuple[SliceableBuffer, SliceableBuffer]: ...

class String8(ValueField):
    structcode: None
    pad: int
    def __init__(self, name: str, pad: int = 1) -> None: ...
    def pack_value(self, val: bytes | str) -> tuple[bytes, int, None]: ...  # type: ignore[override]  # Override Callable
    @overload  # type: ignore[override]  # Overload for specific values
    def parse_binary_value(
        self, data: bytes | bytearray, display: Unused, length: None, format: Unused
    ) -> tuple[str, Literal[b""]]: ...
    @overload
    def parse_binary_value(
        self, data: SliceableBuffer, display: Unused, length: int, format: Unused
    ) -> tuple[str, SliceableBuffer]: ...

class String16(ValueField):
    structcode: None
    pad: int
    def __init__(self, name: str, pad: int = 1) -> None: ...
    def pack_value(self, val: Sequence[object]) -> tuple[bytes, int, None]: ...  # type: ignore[override]  # Override Callable
    def parse_binary_value(  # type: ignore[override]  # length: None will error. See: https://github.com/python-xlib/python-xlib/pull/248
        self, data: SliceableBuffer, display: Unused, length: int | Literal["odd", "even"], format: Unused
    ) -> tuple[tuple[Any, ...], SliceableBuffer]: ...

class List(ValueField):
    structcode: None
    type: Struct | ScalarObj | ResourceObj | ClassInfoClass | type[ValueField]
    pad: int
    def __init__(
        self, name: str, type: Struct | ScalarObj | ResourceObj | ClassInfoClass | Type[ValueField], pad: int = 1
    ) -> None: ...
    def parse_binary_value(
        self, data: SliceableBuffer, display: display.Display | None, length: SupportsIndex | None, format: Unused
    ) -> tuple[list[DictWrapper | None], SliceableBuffer]: ...
    def pack_value(  # type: ignore[override]  # Override Callable
        self, val: Sequence[object] | dict[str, Any]
    ) -> tuple[bytes, int, None]: ...

class FixedList(List):
    size: int
    def __init__(self, name: str, size: int, type: Struct | ScalarObj, pad: int = 1) -> None: ...
    def parse_binary_value(
        self, data: SliceableBuffer, display: display.Display | None, length: Unused, format: Unused
    ) -> tuple[list[DictWrapper | None], SliceableBuffer]: ...

class Object(ValueField):
    type: Struct
    structcode: str | None
    def __init__(self, name: str, type: Struct, default: int | None = None) -> None: ...
    def parse_binary_value(
        self, data: SliceableBuffer, display: display.Display | None, length: Unused, format: Unused
    ) -> tuple[DictWrapper, SliceableBuffer]: ...
    def parse_value(self, val: SliceableBuffer, display: display.Display | None) -> DictWrapper: ...  # type: ignore[override]
    def pack_value(  # type: ignore[override]  # Override Callable
        self, val: tuple[object, ...] | dict[str, Any] | DictWrapper
    ) -> bytes: ...
    def check_value(self, val: tuple[_T, ...] | dict[str, _T] | DictWrapper) -> list[_T]: ...  # type: ignore[override]

class PropertyData(ValueField):
    structcode: None
    def parse_binary_value(
        self, data: SliceableBuffer, display: Unused, length: ConvertibleToInt | None, format: int
    ) -> tuple[tuple[int, SliceableBuffer] | None, SliceableBuffer]: ...
    def pack_value(  # type: ignore[override]  # Override Callable
        self, value: tuple[int, Sequence[float] | Sequence[str]]
    ) -> tuple[bytes, int, Literal[8, 16, 32]]: ...

class FixedPropertyData(PropertyData):
    size: int
    def __init__(self, name: str, size: int) -> None: ...

class ValueList(Field):
    structcode: None
    keyword_args: int
    default: str  # type: ignore[assignment]  # Actually different from base class
    maskcode: bytes
    maskcodelen: int
    fields: list[tuple[Field, int]]
    def __init__(self, name: str, mask: int, pad: int, *fields: Field) -> None: ...
    def pack_value(  # type: ignore[override]  # Override Callable
        self, arg: str | dict[str, Any], keys: dict[str, Any]
    ) -> tuple[bytes, None, None]: ...
    def parse_binary_value(
        self, data: SliceableBuffer, display: display.Display | None, length: Unused, format: Unused
    ) -> tuple[DictWrapper, SliceableBuffer]: ...

class KeyboardMapping(ValueField):
    structcode: None
    def parse_binary_value(
        self, data: SliceableBuffer, display: Unused, length: int | None, format: int
    ) -> tuple[list[int], SliceableBuffer]: ...
    def pack_value(  # type: ignore[override]  # Override Callable
        self, value: Sequence[Sequence[object]]
    ) -> tuple[bytes, int, int]: ...

class ModifierMapping(ValueField):
    structcode: None
    def parse_binary_value(
        self, data: SliceableBuffer, display: Unused, length: Unused, format: int
    ) -> tuple[list[array[int]], SliceableBuffer]: ...
    def pack_value(  # type: ignore[override]  # Override Callable
        self, value: _ModifierMappingList8Elements
    ) -> tuple[bytes, int, int]: ...

class EventField(ValueField):
    structcode: None
    def pack_value(self, value: Event) -> tuple[SliceableBuffer, None, None]: ...  # type: ignore[override]  # Override Callable
    def parse_binary_value(  # type: ignore[override]
        self, data: SliceableBuffer, display: display.Display, length: Unused, format: Unused
    ) -> tuple[Event, SliceableBuffer]: ...

class ScalarObj:
    structcode: str
    structvalues: int
    parse_value: None
    check_value: None
    def __init__(self, code: str) -> None: ...

Card8Obj: ScalarObj
Card16Obj: ScalarObj
Card32Obj: ScalarObj

class ResourceObj:
    structcode: str
    structvalues: int
    class_name: str
    check_value: None
    def __init__(self, class_name: str) -> None: ...
    def parse_value(self, value: int, display: _BaseDisplay) -> int | _ResourceBaseClass: ...

WindowObj: ResourceObj
ColormapObj: ResourceObj

class StrClass:
    structcode: None
    def pack_value(self, val: str) -> bytes: ...
    def parse_binary(self, data: bytes | bytearray, display: Unused) -> tuple[str, bytes | bytearray]: ...

Str: StrClass

class Struct:
    name: str
    check_value: Callable[[Any], Any] | None
    keyword_args: bool
    fields: tuple[Field]
    static_codes: str
    static_values: int
    static_fields: list[Field]
    static_size: int
    var_fields: list[Field]
    structcode: str | None
    structvalues: int
    def __init__(self, *fields: Field) -> None: ...
    def to_binary(self, *varargs: object, **keys: object) -> bytes: ...
    def pack_value(self, value: tuple[object, ...] | dict[str, Any] | DictWrapper) -> bytes: ...
    @overload
    def parse_value(self, val: SliceableBuffer, display: display.Display | None, rawdict: Literal[True]) -> dict[str, Any]: ...
    @overload
    def parse_value(
        self, val: SliceableBuffer, display: display.Display | None, rawdict: Literal[False] = False
    ) -> DictWrapper: ...
    @overload
    def parse_binary(
        self, data: SliceableBuffer, display: display.Display | None, rawdict: Literal[True]
    ) -> tuple[dict[str, Any], SliceableBuffer]: ...
    @overload
    def parse_binary(
        self, data: SliceableBuffer, display: display.Display | None, rawdict: Literal[False] = False
    ) -> tuple[DictWrapper, SliceableBuffer]: ...
    # Structs generate their attributes
    # TODO: Create a specific type-only class for all instances of `Struct`
    @type_check_only
    def __getattr__(self, name: str, /) -> Any: ...

class TextElements8(ValueField):
    string_textitem: Struct
    def pack_value(  # type: ignore[override]  # Override Callable
        self, value: Iterable[Field | str | bytes | tuple[Sequence[object], ...] | dict[str, Sequence[object]] | DictWrapper]
    ) -> tuple[bytes, None, None]: ...
    def parse_binary_value(  # type: ignore[override]  # See: https://github.com/python-xlib/python-xlib/pull/249
        self, data: SliceableBuffer, display: display.Display | None, length: Unused, format: Unused
    ) -> tuple[list[DictWrapper], Literal[""]]: ...

class TextElements16(TextElements8):
    string_textitem: Struct

class GetAttrData:
    # GetAttrData classes get their attributes dynamically
    # TODO: Complete all classes inheriting from GetAttrData
    def __getattr__(self, attr: str) -> Any: ...
    def __setattr__(self, name: str, value: Any, /) -> None: ...

class DictWrapper(GetAttrData):
    def __init__(self, dict: dict[str, Any]) -> None: ...
    def __getitem__(self, key: str) -> object: ...
    def __setitem__(self, key: str, value: object) -> None: ...
    def __delitem__(self, key: str) -> None: ...
    def __setattr__(self, key: str, value: object) -> None: ...

class Request:
    def __init__(
        self, display: _BaseDisplay, onerror: ErrorHandler[object] | None = None, *args: object, **keys: object
    ) -> None: ...

class ReplyRequest(GetAttrData):
    def __init__(self, display: display.Display, defer: bool = False, *args: object, **keys: object) -> None: ...
    def reply(self) -> None: ...

class Event(GetAttrData):
    def __init__(
        self, binarydata: SliceableBuffer | None = None, display: display.Display | None = None, **keys: object
    ) -> None: ...

def call_error_handler(
    handler: Callable[[XError, Request | None], _T], error: XError, request: Request | None
) -> _T | Literal[0]: ...



from _typeshed import Incomplete

from braintree.util.http import Http

class GraphQLClient(Http):
    @staticmethod
    def raise_exception_for_graphql_error(response) -> None: ...
    graphql_headers: dict[str, str]
    def __init__(self, config: Incomplete | None = None, environment: Incomplete | None = None) -> None: ...
    def query(self, definition, variables: Incomplete | None = None, operation_name: Incomplete | None = None): ...



from _typeshed import Incomplete

from influxdb_client.domain.scraper_target_request import ScraperTargetRequest

class ScraperTargetResponse(ScraperTargetRequest):
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(
        self,
        id: Incomplete | None = None,
        org: Incomplete | None = None,
        bucket: Incomplete | None = None,
        links: Incomplete | None = None,
        name: Incomplete | None = None,
        type: Incomplete | None = None,
        url: Incomplete | None = None,
        org_id: Incomplete | None = None,
        bucket_id: Incomplete | None = None,
        allow_insecure: bool = False,
    ) -> None: ...
    @property
    def id(self): ...
    @id.setter
    def id(self, id) -> None: ...
    @property
    def org(self): ...
    @org.setter
    def org(self, org) -> None: ...
    @property
    def bucket(self): ...
    @bucket.setter
    def bucket(self, bucket) -> None: ...
    @property
    def links(self): ...
    @links.setter
    def links(self, links) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



import ctypes
import sys

from serial.tools.list_ports_common import ListPortInfo

if sys.platform == "darwin":
    iokit: ctypes.CDLL
    cf: ctypes.CDLL
    kIOMasterPortDefault: int
    kCFAllocatorDefault: ctypes.c_void_p
    kCFStringEncodingMacRoman: int
    kCFStringEncodingUTF8: int
    kUSBVendorString: str
    kUSBSerialNumberString: str
    io_name_size: int
    KERN_SUCCESS: int
    kern_return_t = ctypes.c_int
    kCFNumberSInt8Type: int
    kCFNumberSInt16Type: int
    kCFNumberSInt32Type: int
    kCFNumberSInt64Type: int

    def get_string_property(device_type: ctypes._CData, property: str) -> str | None: ...
    def get_int_property(device_type: ctypes._CData, property: str, cf_number_type: int) -> int | None: ...
    def IORegistryEntryGetName(device: ctypes._CData) -> str | None: ...
    def IOObjectGetClass(device: ctypes._CData) -> bytes: ...
    def GetParentDeviceByType(device: ctypes._CData, parent_type: str) -> ctypes._CData | None: ...
    def GetIOServicesByType(service_type: str) -> list[ctypes._CData]: ...
    def location_to_string(locationID: int) -> str: ...

    # `SuitableSerialInterface` has required attributes `id: int` and `name: str` but they are not defined on the class
    class SuitableSerialInterface: ...

    def scan_interfaces() -> list[SuitableSerialInterface]: ...
    def search_for_locationID_in_interfaces(serial_interfaces: list[SuitableSerialInterface], locationID: int) -> str | None: ...
    def comports(include_links: bool = ...) -> list[ListPortInfo]: ...



from typing import Any

class DirSync:
    connection: Any
    base: Any
    filter: Any
    attributes: Any
    cookie: Any
    object_security: Any
    ancestors_first: Any
    public_data_only: Any
    incremental_values: Any
    max_length: Any
    hex_guid: Any
    more_results: bool
    def __init__(
        self,
        connection,
        sync_base,
        sync_filter,
        attributes,
        cookie,
        object_security,
        ancestors_first,
        public_data_only,
        incremental_values,
        max_length,
        hex_guid,
    ) -> None: ...
    def loop(self): ...



from _typeshed import Incomplete
from typing import Any

class OAuth2Error(Exception):
    error: Any
    status_code: int
    description: str
    uri: Any
    state: Any
    redirect_uri: Any
    client_id: Any
    scopes: Any
    response_type: Any
    response_mode: Any
    grant_type: Any
    def __init__(
        self,
        description: Incomplete | None = None,
        uri: Incomplete | None = None,
        state: Incomplete | None = None,
        status_code: Incomplete | None = None,
        request: Incomplete | None = None,
    ) -> None: ...
    def in_uri(self, uri): ...
    @property
    def twotuples(self): ...
    @property
    def urlencoded(self): ...
    @property
    def json(self): ...
    @property
    def headers(self): ...

class TokenExpiredError(OAuth2Error):
    error: str

class InsecureTransportError(OAuth2Error):
    error: str
    description: str

class MismatchingStateError(OAuth2Error):
    error: str
    description: str

class MissingCodeError(OAuth2Error):
    error: str

class MissingTokenError(OAuth2Error):
    error: str

class MissingTokenTypeError(OAuth2Error):
    error: str

class FatalClientError(OAuth2Error): ...

class InvalidRequestFatalError(FatalClientError):
    error: str

class InvalidRedirectURIError(InvalidRequestFatalError):
    description: str

class MissingRedirectURIError(InvalidRequestFatalError):
    description: str

class MismatchingRedirectURIError(InvalidRequestFatalError):
    description: str

class InvalidClientIdError(InvalidRequestFatalError):
    description: str

class MissingClientIdError(InvalidRequestFatalError):
    description: str

class InvalidRequestError(OAuth2Error):
    error: str

class MissingResponseTypeError(InvalidRequestError):
    description: str

class MissingCodeChallengeError(InvalidRequestError):
    description: str

class MissingCodeVerifierError(InvalidRequestError):
    description: str

class AccessDeniedError(OAuth2Error):
    error: str

class UnsupportedResponseTypeError(OAuth2Error):
    error: str

class UnsupportedCodeChallengeMethodError(InvalidRequestError):
    description: str

class InvalidScopeError(OAuth2Error):
    error: str

class ServerError(OAuth2Error):
    error: str

class TemporarilyUnavailableError(OAuth2Error):
    error: str

class InvalidClientError(FatalClientError):
    error: str
    status_code: int

class InvalidGrantError(OAuth2Error):
    error: str
    status_code: int

class UnauthorizedClientError(OAuth2Error):
    error: str

class UnsupportedGrantTypeError(OAuth2Error):
    error: str

class UnsupportedTokenTypeError(OAuth2Error):
    error: str

class InvalidTokenError(OAuth2Error):
    error: str
    status_code: int
    description: str

class InsufficientScopeError(OAuth2Error):
    error: str
    status_code: int
    description: str

class ConsentRequired(OAuth2Error):
    error: str

class LoginRequired(OAuth2Error):
    error: str

class CustomOAuth2Error(OAuth2Error):
    error: Any
    def __init__(self, error, *args, **kwargs) -> None: ...

def raise_from_error(error, params: Incomplete | None = None) -> None: ...



from _typeshed import Incomplete

from stripe import api_requestor as api_requestor
from stripe.api_resources.abstract import (
    CreateableAPIResource as CreateableAPIResource,
    ListableAPIResource as ListableAPIResource,
    UpdateableAPIResource as UpdateableAPIResource,
    custom_method as custom_method,
)

class Quote(CreateableAPIResource, ListableAPIResource, UpdateableAPIResource):
    OBJECT_NAME: str
    def accept(self, idempotency_key: str | None = None, **params): ...
    def cancel(self, idempotency_key: str | None = None, **params): ...
    def finalize_quote(self, idempotency_key: str | None = None, **params): ...
    def list_line_items(self, idempotency_key: str | None = None, **params): ...
    def pdf(
        self,
        api_key: Incomplete | None = None,
        api_version: Incomplete | None = None,
        stripe_version: Incomplete | None = None,
        stripe_account: Incomplete | None = None,
        **params,
    ): ...



import datetime
from _typeshed import Unused
from collections import OrderedDict
from collections.abc import Iterator
from re import Match, Pattern
from typing import Any, Final, Literal, overload
from typing_extensions import Never, Self, TypeAlias

_RetType: TypeAlias = type[float | datetime.datetime]
_Expressions: TypeAlias = list[str]  # fixed-length list of 5 or 6 strings

M_ALPHAS: Final[dict[str, int]]
DOW_ALPHAS: Final[dict[str, int]]
ALPHAS: Final[dict[str, int]]
step_search_re: Final[Pattern[str]]
only_int_re: Final[Pattern[str]]

WEEKDAYS: Final[str]
MONTHS: Final[str]
star_or_int_re: Final[Pattern[str]]
special_dow_re: Final[Pattern[str]]
re_star: Final[Pattern[str]]
hash_expression_re: Final[Pattern[str]]
VALID_LEN_EXPRESSION: Final[list[int]]
EXPRESSIONS: dict[tuple[str, bytes], _Expressions]

def timedelta_to_seconds(td: datetime.timedelta) -> float: ...

class CroniterError(ValueError): ...
class CroniterBadTypeRangeError(TypeError): ...
class CroniterBadCronError(CroniterError): ...
class CroniterUnsupportedSyntaxError(CroniterBadCronError): ...
class CroniterBadDateError(CroniterError): ...
class CroniterNotAlphaError(CroniterError): ...

def datetime_to_timestamp(d: datetime.datetime) -> float: ...

class croniter(Iterator[Any]):
    MONTHS_IN_YEAR: Final = 12
    RANGES: Final[tuple[tuple[int, int], tuple[int, int], tuple[int, int], tuple[int, int], tuple[int, int], tuple[int, int]]]
    DAYS: Final[
        tuple[
            Literal[31],
            Literal[28],
            Literal[31],
            Literal[30],
            Literal[31],
            Literal[30],
            Literal[31],
            Literal[31],
            Literal[30],
            Literal[31],
            Literal[30],
            Literal[31],
        ]
    ]
    ALPHACONV: Final[
        tuple[dict[Never, Never], dict[Never, Never], dict[str, str], dict[str, int], dict[str, int], dict[Never, Never]]
    ]
    LOWMAP: Final[
        tuple[dict[Never, Never], dict[Never, Never], dict[int, int], dict[int, int], dict[int, int], dict[Never, Never]]
    ]
    LEN_MEANS_ALL: Final[tuple[int, int, int, int, int, int]]
    bad_length: Final[str]

    tzinfo: datetime.tzinfo | None

    # Initialized to None, but immediately set to a float.
    start_time: float
    dst_start_time: float
    cur: float

    expanded: list[list[str]]
    nth_weekday_of_month: dict[str, set[int]]
    expressions: _Expressions

    def __init__(
        self,
        expr_format: str,
        start_time: float | datetime.datetime | None = None,
        ret_type: _RetType | None = ...,
        day_or: bool = True,
        max_years_between_matches: int | None = None,
        is_prev: bool = False,
        hash_id: str | bytes | None = None,
        implement_cron_bug: bool = False,
    ) -> None: ...
    # Most return value depend on ret_type, which can be passed in both as a method argument and as
    # a constructor argument.
    def get_next(self, ret_type: _RetType | None = None, start_time: float | datetime.datetime | None = None) -> Any: ...
    def get_prev(self, ret_type: _RetType | None = None) -> Any: ...
    def get_current(self, ret_type: _RetType | None = None) -> Any: ...
    def set_current(self, start_time: float | datetime.datetime | None, force: bool = True) -> float: ...
    def __iter__(self) -> Self: ...
    def next(
        self, ret_type: _RetType | None = None, start_time: float | datetime.datetime | None = None, is_prev: bool | None = None
    ) -> Any: ...
    __next__ = next
    def all_next(self, ret_type: _RetType | None = None) -> Iterator[Any]: ...
    def all_prev(self, ret_type: _RetType | None = None) -> Iterator[Any]: ...
    def iter(self, ret_type: _RetType | None = ...) -> Iterator[Any]: ...
    def is_leap(self, year: int) -> bool: ...
    @classmethod
    def expand(cls, expr_format: str, hash_id: bytes | None = None) -> tuple[list[list[str]], dict[str, set[int]]]: ...
    @classmethod
    def is_valid(cls, expression: str, hash_id: bytes | None = None) -> bool: ...
    @classmethod
    def match(cls, cron_expression: str, testdate: float | datetime.datetime | None, day_or: bool = True) -> bool: ...
    @classmethod
    def match_range(
        cls, cron_expression: str, from_datetime: datetime.datetime, to_datetime: datetime.datetime, day_or: bool = True
    ) -> bool: ...

def croniter_range(
    start: float | datetime.datetime,
    stop: float | datetime.datetime,
    expr_format: str,
    ret_type: _RetType | None = None,
    day_or: bool = True,
    exclude_ends: bool = False,
    _croniter: type[croniter] | None = None,
) -> Iterator[Any]: ...

class HashExpander:
    cron: croniter
    def __init__(self, cronit: croniter) -> None: ...
    @overload
    def do(
        self,
        idx: int,
        hash_type: Literal["r"],
        hash_id: None = None,
        range_end: int | None = None,
        range_begin: int | None = None,
    ) -> int: ...
    @overload
    def do(
        self, idx: int, hash_type: str, hash_id: bytes, range_end: int | None = None, range_begin: int | None = None
    ) -> int: ...
    @overload
    def do(
        self, idx: int, hash_type: str = "h", *, hash_id: bytes, range_end: int | None = None, range_begin: int | None = None
    ) -> int: ...
    def match(self, efl: Unused, idx: Unused, expr: str, hash_id: bytes | None = None, **kw: Unused) -> Match[str] | None: ...
    def expand(
        self,
        efl: object,
        idx: int,
        expr: str,
        hash_id: bytes | None = None,
        match: Match[str] | None | Literal[""] = "",
        **kw: object,
    ) -> str: ...

EXPANDERS: OrderedDict[str, HashExpander]



from _typeshed import Incomplete
from typing import ClassVar

from ..core import WesternCalendar

class Italy(WesternCalendar):
    include_labour_day: ClassVar[bool]
    labour_day_label: ClassVar[str]
    FIXED_HOLIDAYS: Incomplete
    include_immaculate_conception: ClassVar[bool]
    include_epiphany: ClassVar[bool]
    include_easter_monday: ClassVar[bool]
    include_assumption: ClassVar[bool]
    include_all_saints: ClassVar[bool]
    include_boxing_day: ClassVar[bool]
    boxing_day_label: ClassVar[str]



from _typeshed import Incomplete

from passlib.crypto.digest import norm_hash_name as norm_hash_name

def get_prf(name): ...
def pbkdf1(secret, salt, rounds, keylen: Incomplete | None = None, hash: str = "sha1"): ...
def pbkdf2(secret, salt, rounds, keylen: Incomplete | None = None, prf: str = "hmac-sha1"): ...



from re import Pattern
from typing import ClassVar

from markdown.blockprocessors import BlockProcessor
from markdown.extensions import Extension
from markdown.inlinepatterns import InlineProcessor

class AbbrExtension(Extension): ...

class AbbrPreprocessor(BlockProcessor):
    RE: ClassVar[Pattern[str]]

class AbbrInlineProcessor(InlineProcessor):
    title: str
    def __init__(self, pattern: str, title: str) -> None: ...

def makeExtension(**kwargs) -> AbbrExtension: ...



import sys

from gevent._config import config as config
from gevent._hub_local import get_hub as get_hub
from gevent._hub_primitives import iwait_on_objects as iwait, wait_on_objects as wait
from gevent.greenlet import Greenlet as Greenlet, joinall as joinall, killall as killall
from gevent.hub import (
    GreenletExit as GreenletExit,
    getcurrent as getcurrent,
    idle as idle,
    kill as kill,
    reinit as reinit,
    signal as signal_handler,
    sleep as sleep,
    spawn_raw as spawn_raw,
)
from gevent.timeout import Timeout as Timeout, with_timeout as with_timeout

if sys.platform != "win32":
    from gevent.os import fork

    __all__ = [
        "Greenlet",
        "GreenletExit",
        "Timeout",
        "config",
        "fork",
        "get_hub",
        "getcurrent",
        "getswitchinterval",
        "idle",
        "iwait",
        "joinall",
        "kill",
        "killall",
        "reinit",
        "setswitchinterval",
        "signal_handler",
        "sleep",
        "spawn",
        "spawn_later",
        "spawn_raw",
        "wait",
        "with_timeout",
    ]
else:
    __all__ = [
        "Greenlet",
        "GreenletExit",
        "Timeout",
        "config",
        "get_hub",
        "getcurrent",
        "getswitchinterval",
        "idle",
        "iwait",
        "joinall",
        "kill",
        "killall",
        "reinit",
        "setswitchinterval",
        "signal_handler",
        "sleep",
        "spawn",
        "spawn_later",
        "spawn_raw",
        "wait",
        "with_timeout",
    ]

__version__: str

getswitchinterval = sys.getswitchinterval
setswitchinterval = sys.setswitchinterval
spawn = Greenlet.spawn
spawn_later = Greenlet.spawn_later



from _typeshed import Incomplete
from collections.abc import Iterable
from typing import Any
from typing_extensions import TypeAlias

from influxdb_client.client._base import _BaseWriteApi
from influxdb_client.client.write.point import Point
from influxdb_client.client.write_api import PointSettings
from influxdb_client.domain.write_precision import _WritePrecision

_DataClass: TypeAlias = Any  # any dataclass
_NamedTuple: TypeAlias = tuple[Any, ...]  # any NamedTuple

logger: Incomplete

class WriteApiAsync(_BaseWriteApi):
    def __init__(self, influxdb_client, point_settings: PointSettings = ...) -> None: ...
    async def write(
        self,
        bucket: str,
        org: str | None = None,
        record: (
            str
            | Iterable[str]
            | Point
            | Iterable[Point]
            | dict[Incomplete, Incomplete]
            | Iterable[dict[Incomplete, Incomplete]]
            | bytes
            | Iterable[bytes]
            | _NamedTuple
            | Iterable[_NamedTuple]
            | _DataClass
            | Iterable[_DataClass]
        ) = None,
        write_precision: _WritePrecision = "ns",
        **kwargs,
    ) -> bool: ...



from typing import Any

class HttpLib2Error(Exception): ...

class HttpLib2ErrorWithResponse(HttpLib2Error):
    response: Any
    content: Any
    def __init__(self, desc, response, content) -> None: ...

class RedirectMissingLocation(HttpLib2ErrorWithResponse): ...
class RedirectLimit(HttpLib2ErrorWithResponse): ...
class FailedToDecompressContent(HttpLib2ErrorWithResponse): ...
class UnimplementedDigestAuthOptionError(HttpLib2ErrorWithResponse): ...
class UnimplementedHmacDigestAuthOptionError(HttpLib2ErrorWithResponse): ...
class MalformedHeader(HttpLib2Error): ...
class RelativeURIError(HttpLib2Error): ...
class ServerNotFoundError(HttpLib2Error): ...
class ProxiesUnavailableError(HttpLib2Error): ...



class PacketTypes:
    indexes: range
    CONNECT: int
    CONNACK: int
    PUBLISH: int
    PUBACK: int
    PUBREC: int
    PUBREL: int
    PUBCOMP: int
    SUBSCRIBE: int
    SUBACK: int
    UNSUBSCRIBE: int
    UNSUBACK: int
    PINGREQ: int
    PINGRESP: int
    DISCONNECT: int
    AUTH: int
    WILLMESSAGE: int
    Names: list[str]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Protocol Buffers - Google's data interchange format
Copyright 2008 Google Inc.  All rights reserved.
https://developers.google.com/protocol-buffers/

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

    * Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above
copyright notice, this list of conditions and the following disclaimer
in the documentation and/or other materials provided with the
distribution.
    * Neither the name of Google Inc. nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""
import builtins
import typing as typing_extensions

import google.protobuf.descriptor
import google.protobuf.internal.well_known_types
import google.protobuf.message

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class Any(google.protobuf.message.Message, google.protobuf.internal.well_known_types.Any):
    """`Any` contains an arbitrary serialized protocol buffer message along with a
    URL that describes the type of the serialized message.

    Protobuf library provides support to pack/unpack Any values in the form
    of utility functions or additional generated methods of the Any type.

    Example 1: Pack and unpack a message in C++.

        Foo foo = ...;
        Any any;
        any.PackFrom(foo);
        ...
        if (any.UnpackTo(&foo)) {
          ...
        }

    Example 2: Pack and unpack a message in Java.

        Foo foo = ...;
        Any any = Any.pack(foo);
        ...
        if (any.is(Foo.class)) {
          foo = any.unpack(Foo.class);
        }

    Example 3: Pack and unpack a message in Python.

        foo = Foo(...)
        any = Any()
        any.Pack(foo)
        ...
        if any.Is(Foo.DESCRIPTOR):
          any.Unpack(foo)
          ...

    Example 4: Pack and unpack a message in Go

         foo := &pb.Foo{...}
         any, err := anypb.New(foo)
         if err != nil {
           ...
         }
         ...
         foo := &pb.Foo{}
         if err := any.UnmarshalTo(foo); err != nil {
           ...
         }

    The pack methods provided by protobuf library will by default use
    'type.googleapis.com/full.type.name' as the type URL and the unpack
    methods only use the fully qualified type name after the last '/'
    in the type URL, for example "foo.bar.com/x/y.z" will yield type
    name "y.z".


    JSON

    The JSON representation of an `Any` value uses the regular
    representation of the deserialized, embedded message, with an
    additional field `@type` which contains the type URL. Example:

        package google.profile;
        message Person {
          string first_name = 1;
          string last_name = 2;
        }

        {
          "@type": "type.googleapis.com/google.profile.Person",
          "firstName": <string>,
          "lastName": <string>
        }

    If the embedded message type is well-known and has a custom JSON
    representation, that representation will be embedded adding a field
    `value` which holds the custom JSON in addition to the `@type`
    field. Example (for message [google.protobuf.Duration][]):

        {
          "@type": "type.googleapis.com/google.protobuf.Duration",
          "value": "1.212s"
        }
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TYPE_URL_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    type_url: builtins.str
    """A URL/resource name that uniquely identifies the type of the serialized
    protocol buffer message. This string must contain at least
    one "/" character. The last segment of the URL's path must represent
    the fully qualified name of the type (as in
    `path/google.protobuf.Duration`). The name should be in a canonical form
    (e.g., leading "." is not accepted).

    In practice, teams usually precompile into the binary all types that they
    expect it to use in the context of Any. However, for URLs which use the
    scheme `http`, `https`, or no scheme, one can optionally set up a type
    server that maps type URLs to message definitions as follows:

    * If no scheme is provided, `https` is assumed.
    * An HTTP GET on the URL must yield a [google.protobuf.Type][]
      value in binary format, or produce an error.
    * Applications are allowed to cache lookup results based on the
      URL, or have them precompiled into a binary to avoid any
      lookup. Therefore, binary compatibility needs to be preserved
      on changes to types. (Use versioned type names to manage
      breaking changes.)

    Note: this functionality is not currently available in the official
    protobuf release, and it is not used for type URLs beginning with
    type.googleapis.com.

    Schemes other than `http`, `https` (or the empty scheme) might be
    used with implementation specific semantics.
    """
    value: builtins.bytes
    """Must be a valid serialized protocol buffer of the above specified type."""
    def __init__(
        self,
        *,
        type_url: builtins.str | None = ...,
        value: builtins.bytes | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["type_url", b"type_url", "value", b"value"]) -> None: ...

global___Any = Any



from typing import Any

from .mockBase import MockBaseStrategy
from .sync import SyncStrategy

class MockSyncStrategy(MockBaseStrategy, SyncStrategy):
    def __init__(self, ldap_connection) -> None: ...
    def post_send_search(self, payload): ...
    bound: Any
    def post_send_single_response(self, payload): ...



from networkx.utils.backends import _dispatch

@_dispatch
def is_graphical(sequence, method: str = "eg"): ...
@_dispatch
def is_valid_degree_sequence_havel_hakimi(deg_sequence): ...
@_dispatch
def is_valid_degree_sequence_erdos_gallai(deg_sequence): ...
@_dispatch
def is_multigraphical(sequence): ...
@_dispatch
def is_pseudographical(sequence): ...
@_dispatch
def is_digraphical(in_sequence, out_sequence): ...



from _typeshed import Incomplete

PY3: bool

class DefusedXmlException(ValueError): ...

class DTDForbidden(DefusedXmlException):
    name: Incomplete
    sysid: Incomplete
    pubid: Incomplete
    def __init__(self, name, sysid, pubid) -> None: ...

class EntitiesForbidden(DefusedXmlException):
    name: Incomplete
    value: Incomplete
    base: Incomplete
    sysid: Incomplete
    pubid: Incomplete
    notation_name: Incomplete
    def __init__(self, name, value, base, sysid, pubid, notation_name) -> None: ...

class ExternalReferenceForbidden(DefusedXmlException):
    context: Incomplete
    base: Incomplete
    sysid: Incomplete
    pubid: Incomplete
    def __init__(self, context, base, sysid, pubid) -> None: ...

class NotSupportedError(DefusedXmlException): ...



from _typeshed import StrOrBytesPath
from typing import Any, overload

from gevent.baseserver import _Spawner
from gevent.server import StreamServer, _Address
from gevent.socket import socket as _GeventSocket
from gevent.ssl import SSLContext

class BackdoorServer(StreamServer):
    locals: dict[str, Any]
    banner: str | None
    @overload
    def __init__(
        self,
        listener: _GeventSocket | tuple[str, int] | str,
        locals: dict[str, Any] | None = None,
        banner: str | None = None,
        *,
        backlog: int | None = None,
        spawn: _Spawner = "default",
        ssl_context: SSLContext,
        server_side: bool = True,
        do_handshake_on_connect: bool = True,
        suppress_ragged_eofs: bool = True,
    ) -> None: ...
    @overload
    def __init__(
        self,
        listener: _GeventSocket | tuple[str, int] | str,
        locals: dict[str, Any] | None = None,
        banner: str | None = None,
        *,
        backlog: int | None = None,
        spawn: _Spawner = "default",
        keyfile: StrOrBytesPath = ...,
        certfile: StrOrBytesPath = ...,
        server_side: bool = True,
        cert_reqs: int = ...,
        ssl_version: int = ...,
        ca_certs: str = ...,
        do_handshake_on_connect: bool = True,
        suppress_ragged_eofs: bool = True,
        ciphers: str = ...,
    ) -> None: ...
    def handle(self, conn: _GeventSocket, _address: _Address) -> None: ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def random_reference(G, niter: int = 1, connectivity: bool = True, seed: Incomplete | None = None): ...
@_dispatch
def lattice_reference(
    G, niter: int = 5, D: Incomplete | None = None, connectivity: bool = True, seed: Incomplete | None = None
): ...
@_dispatch
def sigma(G, niter: int = 100, nrand: int = 10, seed: Incomplete | None = None): ...
@_dispatch
def omega(G, niter: int = 5, nrand: int = 10, seed: Incomplete | None = None): ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def incidence_matrix(
    G,
    nodelist: Incomplete | None = None,
    edgelist: Incomplete | None = None,
    oriented: bool = False,
    weight: Incomplete | None = None,
): ...
@_dispatch
def adjacency_matrix(G, nodelist: Incomplete | None = None, dtype: Incomplete | None = None, weight: str = "weight"): ...



from _typeshed import Incomplete

from antlr4.atn.ATN import ATN as ATN
from antlr4.atn.ATNConfig import ATNConfig as ATNConfig
from antlr4.atn.ATNState import ATNState as ATNState, RuleStopState as RuleStopState
from antlr4.atn.Transition import (
    AbstractPredicateTransition as AbstractPredicateTransition,
    NotSetTransition as NotSetTransition,
    RuleTransition as RuleTransition,
    WildcardTransition as WildcardTransition,
)
from antlr4.IntervalSet import IntervalSet as IntervalSet
from antlr4.PredictionContext import (
    PredictionContext as PredictionContext,
    PredictionContextFromRuleContext as PredictionContextFromRuleContext,
    SingletonPredictionContext as SingletonPredictionContext,
)
from antlr4.RuleContext import RuleContext as RuleContext
from antlr4.Token import Token as Token

class LL1Analyzer:
    HIT_PRED: Incomplete
    atn: Incomplete
    def __init__(self, atn: ATN) -> None: ...
    def getDecisionLookahead(self, s: ATNState): ...
    def LOOK(self, s: ATNState, stopState: ATNState | None = None, ctx: RuleContext | None = None): ...



from _typeshed import ConvertibleToInt, Incomplete
from typing import ClassVar, Literal, overload
from typing_extensions import Self

from openpyxl.descriptors.base import Alias, Bool, Integer, String, _ConvertibleToBool
from openpyxl.descriptors.serialisable import Serialisable

from ..xml._functions_overloads import _SupportsIterAndAttribAndTextAndGet

class WorkbookProtection(Serialisable):
    tagname: ClassVar[str]
    workbook_password: Alias
    workbookPasswordCharacterSet: String[Literal[True]]
    revision_password: Alias
    revisionsPasswordCharacterSet: String[Literal[True]]
    lockStructure: Bool[Literal[True]]
    lock_structure: Alias
    lockWindows: Bool[Literal[True]]
    lock_windows: Alias
    lockRevision: Bool[Literal[True]]
    lock_revision: Alias
    revisionsAlgorithmName: String[Literal[True]]
    revisionsHashValue: Incomplete
    revisionsSaltValue: Incomplete
    revisionsSpinCount: Integer[Literal[True]]
    workbookAlgorithmName: String[Literal[True]]
    workbookHashValue: Incomplete
    workbookSaltValue: Incomplete
    workbookSpinCount: Integer[Literal[True]]
    __attrs__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        workbookPassword: Incomplete | None = None,
        workbookPasswordCharacterSet: str | None = None,
        revisionsPassword: Incomplete | None = None,
        revisionsPasswordCharacterSet: str | None = None,
        lockStructure: _ConvertibleToBool | None = None,
        lockWindows: _ConvertibleToBool | None = None,
        lockRevision: _ConvertibleToBool | None = None,
        revisionsAlgorithmName: str | None = None,
        revisionsHashValue: Incomplete | None = None,
        revisionsSaltValue: Incomplete | None = None,
        revisionsSpinCount: ConvertibleToInt | None = None,
        workbookAlgorithmName: str | None = None,
        workbookHashValue: Incomplete | None = None,
        workbookSaltValue: Incomplete | None = None,
        workbookSpinCount: ConvertibleToInt | None = None,
    ) -> None: ...
    @overload
    def set_workbook_password(self, value: str = "", already_hashed: Literal[False] = False) -> None: ...
    @overload
    def set_workbook_password(self, value: str | None, already_hashed: Literal[True]) -> None: ...
    @overload
    def set_workbook_password(self, value: str | None = "", *, already_hashed: Literal[True]) -> None: ...
    @property
    def workbookPassword(self) -> str | None: ...
    @workbookPassword.setter
    def workbookPassword(self, value: str) -> None: ...
    @overload
    def set_revisions_password(self, value: str = "", already_hashed: Literal[False] = False) -> None: ...
    @overload
    def set_revisions_password(self, value: str | None, already_hashed: Literal[True]) -> None: ...
    @overload
    def set_revisions_password(self, value: str | None = "", *, already_hashed: Literal[True]) -> None: ...
    @property
    def revisionsPassword(self) -> str | None: ...
    @revisionsPassword.setter
    def revisionsPassword(self, value: str) -> None: ...
    @classmethod
    def from_tree(cls, node: _SupportsIterAndAttribAndTextAndGet) -> Self: ...

DocumentSecurity = WorkbookProtection

class FileSharing(Serialisable):
    tagname: ClassVar[str]
    readOnlyRecommended: Bool[Literal[True]]
    userName: String[Literal[True]]
    reservationPassword: Incomplete
    algorithmName: String[Literal[True]]
    hashValue: Incomplete
    saltValue: Incomplete
    spinCount: Integer[Literal[True]]
    def __init__(
        self,
        readOnlyRecommended: _ConvertibleToBool | None = None,
        userName: str | None = None,
        reservationPassword: Incomplete | None = None,
        algorithmName: str | None = None,
        hashValue: Incomplete | None = None,
        saltValue: Incomplete | None = None,
        spinCount: ConvertibleToInt | None = None,
    ) -> None: ...



from _typeshed import Incomplete
from typing import ClassVar, Final

class ScriptLanguage:
    FLUX: Final = "flux"
    SQL: Final = "sql"
    INFLUXQL: Final = "influxql"

    openapi_types: ClassVar[dict[Incomplete, Incomplete]]
    attribute_map: ClassVar[dict[Incomplete, Incomplete]]
    def to_dict(self) -> dict[Incomplete, Incomplete]: ...
    def to_str(self) -> str: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...



from os import _Environ

InterruptedError: type
string_types: tuple[type, ...]

def is_executable_file(path): ...
def which(filename, env: _Environ[str] | None = None): ...
def split_command_line(command_line): ...
def select_ignore_interrupts(iwtd, owtd, ewtd, timeout: float | None = None): ...
def poll_ignore_interrupts(fds, timeout: float | None = None): ...
